#!/usr/bin/python
# (c) 2019, XLAB d.o.o <www.xlab.si>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type


DOCUMENTATION = r'''
---
module: s3_bucket_notification
version_added: 1.0.0
short_description: Creates, updates or deletes S3 Bucket notification for lambda
description:
    - This module allows the management of AWS Lambda function bucket event mappings via the
      Ansible framework. Use module M(community.aws.lambda) to manage the lambda function itself, M(community.aws.lambda_alias)
      to manage function aliases and M(community.aws.lambda_policy) to modify lambda permissions.
notes:
    - This module heavily depends on M(community.aws.lambda_policy) as you need to allow C(lambda:InvokeFunction)
       permission for your lambda function.

author:
    - XLAB d.o.o. (@xlab-si)
    - Aljaz Kosir (@aljazkosir)
    - Miha Plesko (@miha-plesko)
options:
  event_name:
    description:
      - Unique name for event notification on bucket.
    required: true
    type: str
  lambda_function_arn:
    description:
      - The ARN of the lambda function.
    aliases: ['function_arn']
    type: str
  bucket_name:
    description:
      - S3 bucket name.
    required: true
    type: str
  state:
    description:
      - Describes the desired state.
    default: "present"
    choices: ["present", "absent"]
    type: str
  lambda_alias:
    description:
      - Name of the Lambda function alias.
      - Mutually exclusive with I(lambda_version).
    type: str
  lambda_version:
    description:
      - Version of the Lambda function.
      - Mutually exclusive with I(lambda_alias).
    type: int
  events:
    description:
      - Events that you want to be triggering notifications. You can select multiple events to send
        to the same destination, you can set up different events to send to different destinations,
        and you can set up a prefix or suffix for an event. However, for each bucket,
        individual events cannot have multiple configurations with overlapping prefixes or
        suffixes that could match the same object key.
      - Required when I(state=present).
    choices: ['s3:ObjectCreated:*', 's3:ObjectCreated:Put', 's3:ObjectCreated:Post',
              's3:ObjectCreated:Copy', 's3:ObjectCreated:CompleteMultipartUpload',
              's3:ObjectRemoved:*', 's3:ObjectRemoved:Delete',
              's3:ObjectRemoved:DeleteMarkerCreated', 's3:ObjectRestore:Post',
              's3:ObjectRestore:Completed', 's3:ReducedRedundancyLostObject']
    type: list
    elements: str
  prefix:
    description:
      - Optional prefix to limit the notifications to objects with keys that start with matching
        characters.
    type: str
  suffix:
    description:
      - Optional suffix to limit the notifications to objects with keys that end with matching
        characters.
    type: str
requirements:
    - boto3
extends_documentation_fragment:
- amazon.aws.aws
- amazon.aws.ec2

'''

EXAMPLES = r'''
---
# Example that creates a lambda event notification for a bucket
- name: Process jpg image
  community.aws.s3_bucket_notification:
    state: present
    event_name: on_file_add_or_remove
    bucket_name: test-bucket
    function_name: arn:aws:lambda:us-east-2:526810320200:function:test-lambda
    events: ["s3:ObjectCreated:*", "s3:ObjectRemoved:*"]
    prefix: images/
    suffix: .jpg
'''

RETURN = r'''
notification_configuration:
    description: list of currently applied notifications
    returned: success
    type: list
'''

from ansible_collections.amazon.aws.plugins.module_utils.core import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import camel_dict_to_snake_dict

try:
    from botocore.exceptions import ClientError, BotoCoreError
except ImportError:
    pass  # will be protected by AnsibleAWSModule


class AmazonBucket:
    def __init__(self, client, bucket_name):
        self.client = client
        self.bucket_name = bucket_name
        self._full_config_cache = None

    def full_config(self):
        if self._full_config_cache is None:
            self._full_config_cache = [Config.from_api(cfg) for cfg in
                                       self.client.get_bucket_notification_configuration(
                                           Bucket=self.bucket_name).get(
                                           'LambdaFunctionConfigurations', list())]
        return self._full_config_cache

    def current_config(self, config_name):
        for config in self.full_config():
            if config.raw['Id'] == config_name:
                return config

    def apply_config(self, desired):
        configs = [cfg.raw for cfg in self.full_config() if cfg.name != desired.raw['Id']]
        configs.append(desired.raw)
        self._upload_bucket_config(configs)
        return configs

    def delete_config(self, desired):
        configs = [cfg.raw for cfg in self.full_config() if cfg.name != desired.raw['Id']]
        self._upload_bucket_config(configs)
        return configs

    def _upload_bucket_config(self, config):
        self.client.put_bucket_notification_configuration(
            Bucket=self.bucket_name,
            NotificationConfiguration={
                'LambdaFunctionConfigurations': config
            })


class Config:
    def __init__(self, content):
        self._content = content
        self.name = content['Id']

    @property
    def raw(self):
        return self._content

    def __eq__(self, other):
        if other:
            return self.raw == other.raw
        return False

    @classmethod
    def from_params(cls, **params):
        function_arn = params['lambda_function_arn']

        qualifier = None
        if params['lambda_version'] > 0:
            qualifier = str(params['lambda_version'])
        elif params['lambda_alias']:
            qualifier = str(params['lambda_alias'])
        if qualifier:
            params['lambda_function_arn'] = '{0}:{1}'.format(function_arn, qualifier)

        return cls({
            'Id': params['event_name'],
            'LambdaFunctionArn': params['lambda_function_arn'],
            'Events': sorted(params['events']),
            'Filter': {
                'Key': {
                    'FilterRules': [{
                        'Name': 'Prefix',
                        'Value': params['prefix']
                    }, {
                        'Name': 'Suffix',
                        'Value': params['suffix']
                    }]
                }
            }
        })

    @classmethod
    def from_api(cls, config):
        return cls(config)


def main():
    event_types = ['s3:ObjectCreated:*', 's3:ObjectCreated:Put', 's3:ObjectCreated:Post',
                   's3:ObjectCreated:Copy', 's3:ObjectCreated:CompleteMultipartUpload',
                   's3:ObjectRemoved:*', 's3:ObjectRemoved:Delete',
                   's3:ObjectRemoved:DeleteMarkerCreated', 's3:ObjectRestore:Post',
                   's3:ObjectRestore:Completed', 's3:ReducedRedundancyLostObject']
    argument_spec = dict(
        state=dict(default='present', choices=['present', 'absent']),
        event_name=dict(required=True),
        lambda_function_arn=dict(aliases=['function_arn']),
        bucket_name=dict(required=True),
        events=dict(type='list', default=[], choices=event_types, elements='str'),
        prefix=dict(default=''),
        suffix=dict(default=''),
        lambda_alias=dict(),
        lambda_version=dict(type='int', default=0),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[['lambda_alias', 'lambda_version']],
        required_if=[['state', 'present', ['events']]]
    )

    bucket = AmazonBucket(module.client('s3'), module.params['bucket_name'])
    current = bucket.current_config(module.params['event_name'])
    desired = Config.from_params(**module.params)
    notification_configuration = [cfg.raw for cfg in bucket.full_config()]

    state = module.params['state']
    try:
        if (state == 'present' and current == desired) or (state == 'absent' and not current):
            changed = False
        elif module.check_mode:
            changed = True
        elif state == 'present':
            changed = True
            notification_configuration = bucket.apply_config(desired)
        elif state == 'absent':
            changed = True
            notification_configuration = bucket.delete_config(desired)
    except (ClientError, BotoCoreError) as e:
        module.fail_json(msg='{0}'.format(e))

    module.exit_json(**dict(changed=changed,
                            notification_configuration=[camel_dict_to_snake_dict(cfg) for cfg in
                                                        notification_configuration]))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''

module: pingdom
short_description: Pause/unpause Pingdom alerts
description:
    - This module will let you pause/unpause Pingdom alerts
author:
    - "Dylan Silva (@thaumos)"
    - "Justin Johns (!UNKNOWN)"
requirements:
    - "This pingdom python library: https://github.com/mbabineau/pingdom-python"
options:
    state:
        type: str
        description:
            - Define whether or not the check should be running or paused.
        required: true
        choices: [ "running", "paused", "started", "stopped" ]
    checkid:
        type: str
        description:
            - Pingdom ID of the check.
        required: true
    uid:
        type: str
        description:
            - Pingdom user ID.
        required: true
    passwd:
        type: str
        description:
            - Pingdom user password.
        required: true
    key:
        type: str
        description:
            - Pingdom API key.
        required: true
notes:
    - This module does not yet have support to add/remove checks.
'''

EXAMPLES = '''
- name: Pause the check with the ID of 12345
  community.general.pingdom:
    uid: example@example.com
    passwd: password123
    key: apipassword123
    checkid: 12345
    state: paused

- name: Unpause the check with the ID of 12345
  community.general.pingdom:
    uid: example@example.com
    passwd: password123
    key: apipassword123
    checkid: 12345
    state: running
'''

import traceback

PINGDOM_IMP_ERR = None
try:
    import pingdom
    HAS_PINGDOM = True
except Exception:
    PINGDOM_IMP_ERR = traceback.format_exc()
    HAS_PINGDOM = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib


def pause(checkid, uid, passwd, key):

    c = pingdom.PingdomConnection(uid, passwd, key)
    c.modify_check(checkid, paused=True)
    check = c.get_check(checkid)
    name = check.name
    result = check.status
    # if result != "paused":             # api output buggy - accept raw exception for now
    #    return (True, name, result)
    return (False, name, result)


def unpause(checkid, uid, passwd, key):

    c = pingdom.PingdomConnection(uid, passwd, key)
    c.modify_check(checkid, paused=False)
    check = c.get_check(checkid)
    name = check.name
    result = check.status
    # if result != "up":                 # api output buggy - accept raw exception for now
    #    return (True, name, result)
    return (False, name, result)


def main():

    module = AnsibleModule(
        argument_spec=dict(
            state=dict(required=True, choices=['running', 'paused', 'started', 'stopped']),
            checkid=dict(required=True),
            uid=dict(required=True),
            passwd=dict(required=True, no_log=True),
            key=dict(required=True, no_log=True),
        )
    )

    if not HAS_PINGDOM:
        module.fail_json(msg=missing_required_lib("pingdom"), exception=PINGDOM_IMP_ERR)

    checkid = module.params['checkid']
    state = module.params['state']
    uid = module.params['uid']
    passwd = module.params['passwd']
    key = module.params['key']

    if (state == "paused" or state == "stopped"):
        (rc, name, result) = pause(checkid, uid, passwd, key)

    if (state == "running" or state == "started"):
        (rc, name, result) = unpause(checkid, uid, passwd, key)

    if rc != 0:
        module.fail_json(checkid=checkid, name=name, status=result)

    module.exit_json(checkid=checkid, name=name, status=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Ansible Project
# Copyright: (c) 2020, Chocolatey Software
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: win_chocolatey_feature
version_added: '2.7'
short_description: Manages Chocolatey features
description:
- Used to enable or disable features in Chocolatey.
options:
  name:
    description:
    - The name of the feature to manage.
    - Run C(choco.exe feature list) to get a list of features that can be
      managed.
    type: str
    required: yes
  state:
    description:
    - When C(disabled) then the feature will be disabled.
    - When C(enabled) then the feature will be enabled.
    type: str
    choices: [ disabled, enabled ]
    default: enabled
seealso:
- module: win_chocolatey
- module: win_chocolatey_config
- module: win_chocolatey_facts
- module: win_chocolatey_source
author:
- Jordan Borean (@jborean93)
'''

EXAMPLES = r'''
- name: Disable file checksum matching
  win_chocolatey_feature:
    name: checksumFiles
    state: disabled

- name: Stop Chocolatey on the first package failure
  win_chocolatey_feature:
    name: stopOnFirstPackageFailure
    state: enabled
'''

RETURN = r'''
'''



import os

import testinfra.utils.ansible_runner

testinfra_hosts = testinfra.utils.ansible_runner.AnsibleRunner(
    os.environ['MOLECULE_INVENTORY_FILE']
).get_hosts('all')


def test_mongod_available(host):
    cmd = host.run("mongod --version")
    assert cmd.rc == 0
    assert "db version" in cmd.stdout


def test_mongo_available(host):
    cmd = host.run("mongo --version")
    assert cmd.rc == 0
    assert "MongoDB shell version" in cmd.stdout


def test_mongos_available(host):
    cmd = host.run("mongos --version")
    assert cmd.rc == 0
    assert "mongos version" in cmd.stdout


def test_mongodump_available(host):
    cmd = host.run("mongodump --version")
    assert cmd.rc == 0
    assert "mongodump version" in cmd.stdout


def test_specific_mongodb_version(host):
    cmd = host.run("mongod --version")
    assert cmd.rc == 0
    assert "4.2.2" in cmd.stdout



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright 2013 Bruce Pennypacker <bruce@pennypacker.org>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: airbrake_deployment
author:
- "Bruce Pennypacker (@bpennypacker)"
- "Patrick Humpal (@phumpal)"
short_description: Notify airbrake about app deployments
description:
   - Notify airbrake about app deployments (see U(https://airbrake.io/docs/api/#deploys-v4)).
   - Parameter I(token) has been deprecated for community.general 0.2.0. Please remove entry.
options:
  project_id:
    description:
      - Airbrake PROJECT_ID
    required: false
    type: str
    version_added: '0.2.0'
  project_key:
    description:
      - Airbrake PROJECT_KEY.
    required: false
    type: str
    version_added: '0.2.0'
  environment:
    description:
      - The airbrake environment name, typically 'production', 'staging', etc.
    required: true
    type: str
  user:
    description:
      - The username of the person doing the deployment
    required: false
    type: str
  repo:
    description:
      - URL of the project repository
    required: false
    type: str
  revision:
    description:
      - A hash, number, tag, or other identifier showing what revision from version control was deployed
    required: false
    type: str
  version:
    description:
      - A string identifying what version was deployed
    required: false
    type: str
    version_added: '1.0.0'
  url:
    description:
      - Optional URL to submit the notification to. Use to send notifications to Airbrake-compliant tools like Errbit.
    required: false
    default: "https://api.airbrake.io/api/v4/projects/"
    type: str
  validate_certs:
    description:
      - If C(no), SSL certificates for the target url will not be validated. This should only be used
        on personally controlled sites using self-signed certificates.
    required: false
    default: 'yes'
    type: bool
  token:
     description:
      - This parameter (API token) has been deprecated in community.general 0.2.0. Please remove it from your tasks.
     required: false
     type: str

requirements: []
'''

EXAMPLES = '''
- name: Notify airbrake about an app deployment
  community.general.airbrake_deployment:
    project_id: '12345'
    project_key: 'AAAAAA'
    environment: staging
    user: ansible
    revision: '4.2'

- name: Notify airbrake about an app deployment, using git hash as revision
  community.general.airbrake_deployment:
    project_id: '12345'
    project_key: 'AAAAAA'
    environment: staging
    user: ansible
    revision: 'e54dd3a01f2c421b558ef33b5f79db936e2dcf15'
    version: '0.2.0'
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url
from ansible.module_utils.six.moves.urllib.parse import urlencode


# ===========================================
# Module execution.
#

def main():

    module = AnsibleModule(
        argument_spec=dict(
            token=dict(required=False, no_log=True, type='str'),
            project_id=dict(required=False, no_log=True, type='str'),
            project_key=dict(required=False, no_log=True, type='str'),
            environment=dict(required=True, type='str'),
            user=dict(required=False, type='str'),
            repo=dict(required=False, type='str'),
            revision=dict(required=False, type='str'),
            version=dict(required=False, type='str'),
            url=dict(required=False, default='https://api.airbrake.io/api/v4/projects/', type='str'),
            validate_certs=dict(default=True, type='bool'),
        ),
        supports_check_mode=True,
        required_together=[('project_id', 'project_key')],
        mutually_exclusive=[('project_id', 'token')],
    )

    # Build list of params
    params = {}

    # If we're in check mode, just exit pretending like we succeeded
    if module.check_mode:
        module.exit_json(changed=True)

    if module.params["token"]:
        # v2 API documented at https://airbrake.io/docs/legacy-xml-api/#tracking-deploys
        if module.params["environment"]:
            params["deploy[rails_env]"] = module.params["environment"]

        if module.params["user"]:
            params["deploy[local_username]"] = module.params["user"]

        if module.params["repo"]:
            params["deploy[scm_repository]"] = module.params["repo"]

        if module.params["revision"]:
            params["deploy[scm_revision]"] = module.params["revision"]

        # version not supported in v2 API; omit

        module.deprecate("Parameter 'token' is deprecated since community.general 0.2.0. Please remove "
                         "it and use 'project_id' and 'project_key' instead",
                         version='3.0.0', collection_name='community.general')  # was Ansible 2.14

        params["api_key"] = module.params["token"]

        # Allow sending to Airbrake compliant v2 APIs
        if module.params["url"] == 'https://api.airbrake.io/api/v4/projects/':
            url = 'https://api.airbrake.io/deploys.txt'
        else:
            url = module.params["url"]

        # Send the data to airbrake
        data = urlencode(params)
        response, info = fetch_url(module, url, data=data)

    if module.params["project_id"] and module.params["project_key"]:
        # v4 API documented at https://airbrake.io/docs/api/#create-deploy-v4
        if module.params["environment"]:
            params["environment"] = module.params["environment"]

        if module.params["user"]:
            params["username"] = module.params["user"]

        if module.params["repo"]:
            params["repository"] = module.params["repo"]

        if module.params["revision"]:
            params["revision"] = module.params["revision"]

        if module.params["version"]:
            params["version"] = module.params["version"]

        # Build deploy url
        url = module.params.get('url') + module.params["project_id"] + '/deploys?key=' + module.params["project_key"]
        json_body = module.jsonify(params)

        # Build header
        headers = {'Content-Type': 'application/json'}

        # Notify Airbrake of deploy
        response, info = fetch_url(module, url, data=json_body,
                                   headers=headers, method='POST')

    if info['status'] == 200 or info['status'] == 201:
        module.exit_json(changed=True)
    else:
        module.fail_json(msg="HTTP result code: %d connecting to %s" % (info['status'], url))


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# (c) 2020 Matt Clay <mclay@redhat.com>
# (c) 2020 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# This module properly lives in community.docker; it has been vendored here
# to support the 1.x.y docker integration tests.

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: current_container_facts
short_description: Return facts about whether the module runs in a Docker container
description:
  - Return facts about whether the module runs in a Docker container.
author:
  - Felix Fontein (@felixfontein)
'''

EXAMPLES = '''
- name: Get facts on current container
  community.docker.current_container_facts:
- name: Print information on current container when running in a container
  ansible.builtin.debug:
    msg: "Container ID is {{ ansible_module_container_id }}"
  when: ansible_module_running_in_container
'''

RETURN = '''
ansible_facts:
    description: Ansible facts returned by the module
    type: dict
    returned: always
    contains:
        ansible_module_running_in_container:
            description:
              - Whether the module was able to detect that it runs in a container or not.
            returned: always
            type: bool
        ansible_module_container_id:
            description:
              - The detected container ID.
              - Contains an empty string if no container was detected.
            returned: always
            type: str
        ansible_module_container_type:
            description:
              - The detected container environment.
              - Contains an empty string if no container was detected.
              - Otherwise, will be one of C(docker) or C(azure_pipelines).
            returned: always
            type: str
            # choices:
            #   - docker
            #   - azure_pipelines
'''

import os

from ansible.module_utils.basic import AnsibleModule


def main():
    module = AnsibleModule(dict(), supports_check_mode=True)

    path = '/proc/self/cpuset'
    container_id = ''
    container_type = ''

    if os.path.exists(path):
        # File content varies based on the environment:
        #   No Container: /
        #   Docker: /docker/c86f3732b5ba3d28bb83b6e14af767ab96abbc52de31313dcb1176a62d91a507
        #   Azure Pipelines (Docker): /azpl_job/0f2edfed602dd6ec9f2e42c867f4d5ee640ebf4c058e6d3196d4393bb8fd0891
        #   Podman: /../../../../../..
        with open(path, 'rb') as f:
            contents = f.read().decode('utf-8')

        cgroup_path, cgroup_name = os.path.split(contents.strip())

        if cgroup_path == '/docker':
            container_id = cgroup_name
            container_type = 'docker'

        if cgroup_path == '/azpl_job':
            container_id = cgroup_name
            container_type = 'azure_pipelines'

    module.exit_json(ansible_facts=dict(
        ansible_module_running_in_container=container_id != '',
        ansible_module_container_id=container_id,
        ansible_module_container_type=container_type,
    ))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# A bare-minimum Ansible Module based on AnsibleAWSModule used for testing some
# of the core behaviour around AWS/Boto3 connection details

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


try:
    import boto.ec2
except ImportError:
    pass

from ansible_collections.amazon.aws.plugins.module_utils.core import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ec2_connect


def main():
    module = AnsibleAWSModule(
        argument_spec={},
        supports_check_mode=True,
        check_boto3=False,
    )

    try:
        client = ec2_connect(module)
    except boto.exception.NoAuthHandlerFound as e:
        module.fail_json_aws(e, msg='Failed to get connection')

    filters = {'name': 'amzn2-ami-hvm-2.0.202006*-x86_64-gp2'}

    try:
        images = client.get_all_images(image_ids=[], filters=filters, owners=['amazon'], executable_by=[])
    except boto.exception.BotoServerError as e:
        module.fail_json_aws(e, msg='Fail JSON AWS')

    images_out = []
    for image in images:
        images_out.append(image.id)

    # Return something, just because we can.
    module.exit_json(
        changed=False,
        images=images_out)


if __name__ == '__main__':
    main()



#
# Copyright (c) 2017 Cisco and/or its affiliates.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import, division, print_function
__metaclass__ = type

import json

from ansible_collections.cisco.nso.tests.unit.compat.mock import patch
from ansible_collections.cisco.nso.plugins.modules import nso_verify
from . import nso_module
from .nso_module import MockResponse

from ansible_collections.cisco.nso.tests.unit.plugins.modules.utils import set_module_args


class TestNsoVerify(nso_module.TestNsoModule):
    module = nso_verify

    @patch('ansible_collections.cisco.nso.plugins.module_utils.nso.open_url')
    def test_nso_verify_empty_data(self, open_url_mock):
        calls = [
            MockResponse('login', {}, 200, '{}', {'set-cookie': 'id'}),
            MockResponse('get_system_setting', {'operation': 'version'}, 200, '{"result": "4.4.3"}'),
            MockResponse('logout', {}, 200, '{"result": {}}'),
        ]
        open_url_mock.side_effect = lambda *args, **kwargs: nso_module.mock_call(calls, *args, **kwargs)

        data = {}
        set_module_args({
            'username': 'user', 'password': 'password',
            'url': 'http://localhost:8080/jsonrpc', 'data': data
        })
        self.execute_module(changed=False)

        self.assertEqual(0, len(calls))

    @patch('ansible_collections.cisco.nso.plugins.module_utils.nso.open_url')
    def test_nso_verify_violation(self, open_url_mock):
        devices_schema = nso_module.load_fixture('devices_schema.json')
        device_schema = nso_module.load_fixture('device_schema.json')
        description_schema = nso_module.load_fixture('description_schema.json')

        calls = [
            MockResponse('login', {}, 200, '{}', {'set-cookie': 'id'}),
            MockResponse('get_system_setting', {'operation': 'version'}, 200, '{"result": "4.5.0"}'),
            MockResponse('get_module_prefix_map', {}, 200, '{"result": {"tailf-ncs": "ncs"}}'),
            MockResponse('new_trans', {'mode': 'read'}, 200, '{"result": {"th": 1}}'),
            MockResponse('get_schema', {'path': '/ncs:devices'}, 200, '{"result": %s}' % (json.dumps(devices_schema, ))),
            MockResponse('get_schema', {'path': '/ncs:devices/device'}, 200, '{"result": %s}' % (json.dumps(device_schema, ))),
            MockResponse('exists', {'path': '/ncs:devices/device{ce0}'}, 200, '{"result": {"exists": true}}'),
            MockResponse('get_value', {'path': '/ncs:devices/device{ce0}/description'}, 200, '{"result": {"value": "In Violation"}}'),
            MockResponse('get_schema', {'path': '/ncs:devices/device/description'}, 200, '{"result": %s}' % (json.dumps(description_schema, ))),
            MockResponse('logout', {}, 200, '{"result": {}}'),
        ]
        open_url_mock.side_effect = lambda *args, **kwargs: nso_module.mock_call(calls, *args, **kwargs)

        data = nso_module.load_fixture('verify_violation_data.json')
        set_module_args({
            'username': 'user', 'password': 'password',
            'url': 'http://localhost:8080/jsonrpc', 'data': data
        })
        self.execute_module(failed=True, violations=[
            {'path': '/ncs:devices/device{ce0}/description', 'expected-value': 'Example Device', 'value': 'In Violation'},
        ])

        self.assertEqual(0, len(calls))

    @patch('ansible_collections.cisco.nso.plugins.module_utils.nso.open_url')
    def test_nso_verify_ok(self, open_url_mock):
        devices_schema = nso_module.load_fixture('devices_schema.json')
        device_schema = nso_module.load_fixture('device_schema.json')

        calls = [
            MockResponse('login', {}, 200, '{}', {'set-cookie': 'id'}),
            MockResponse('get_system_setting', {'operation': 'version'}, 200, '{"result": "4.5.0"}'),
            MockResponse('get_module_prefix_map', {}, 200, '{"result": {"tailf-ncs": "ncs"}}'),
            MockResponse('new_trans', {'mode': 'read'}, 200, '{"result": {"th": 1}}'),
            MockResponse('get_schema', {'path': '/ncs:devices'}, 200, '{"result": %s}' % (json.dumps(devices_schema, ))),
            MockResponse('get_schema', {'path': '/ncs:devices/device'}, 200, '{"result": %s}' % (json.dumps(device_schema, ))),
            MockResponse('exists', {'path': '/ncs:devices/device{ce0}'}, 200, '{"result": {"exists": true}}'),
            MockResponse('get_value', {'path': '/ncs:devices/device{ce0}/description'}, 200, '{"result": {"value": "Example Device"}}'),
            MockResponse('logout', {}, 200, '{"result": {}}'),
        ]
        open_url_mock.side_effect = lambda *args, **kwargs: nso_module.mock_call(calls, *args, **kwargs)

        data = nso_module.load_fixture('verify_violation_data.json')
        set_module_args({
            'username': 'user', 'password': 'password',
            'url': 'http://localhost:8080/jsonrpc', 'data': data,
            'validate_certs': False
        })
        self.execute_module(changed=False)

        self.assertEqual(0, len(calls))



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: ce_reboot
short_description: Reboot a HUAWEI CloudEngine switches.
description:
    - Reboot a HUAWEI CloudEngine switches.
author: Gong Jianjun (@QijunPan)
notes:
    - This module requires the netconf system service be enabled on the remote device being managed.
    - Recommended connection is C(netconf).
    - This module also works with C(local) connections for legacy playbooks.
requirements: ["ncclient"]
options:
    confirm:
        description:
            - Safeguard boolean. Set to true if you're sure you want to reboot.
        type: bool
        required: true
    save_config:
        description:
            - Flag indicating whether to save the configuration.
        required: false
        type: bool
        default: false
'''

EXAMPLES = '''
- name: Reboot module test
  hosts: cloudengine
  connection: local
  gather_facts: no
  vars:
    cli:
      host: "{{ inventory_hostname }}"
      port: "{{ ansible_ssh_port }}"
      username: "{{ username }}"
      password: "{{ password }}"
      transport: cli

  tasks:
  - name: Reboot the device
    community.network.ce_reboot:
      confirm: true
      save_config: true
      provider: "{{ cli }}"
'''

RETURN = '''
rebooted:
    description: Whether the device was instructed to reboot.
    returned: success
    type: bool
    sample: true
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.cloudengine.ce import execute_nc_action, ce_argument_spec

try:
    from ncclient.operations.errors import TimeoutExpiredError
    HAS_NCCLIENT = True
except ImportError:
    HAS_NCCLIENT = False

CE_NC_XML_EXECUTE_REBOOT = """
    <action>
      <devm xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <reboot>
            <saveConfig>%s</saveConfig>
        </reboot>
      </devm>
    </action>
"""


class Reboot(object):
    """ Reboot a network device """

    def __init__(self, **kwargs):
        """ __init___ """

        self.network_module = None
        self.netconf = None
        self.init_network_module(**kwargs)

        self.confirm = self.network_module.params['confirm']
        self.save_config = self.network_module.params['save_config']

    def init_network_module(self, **kwargs):
        """ init network module """

        self.network_module = AnsibleModule(**kwargs)

    def netconf_set_action(self, xml_str):
        """ netconf execute action """

        try:
            execute_nc_action(self.network_module, xml_str)
        except TimeoutExpiredError:
            pass

    def work(self):
        """ start to work """

        if not self.confirm:
            self.network_module.fail_json(
                msg='Error: Confirm must be set to true for this module to work.')

        xml_str = CE_NC_XML_EXECUTE_REBOOT % str(self.save_config).lower()
        self.netconf_set_action(xml_str)


def main():
    """ main """

    argument_spec = dict(
        confirm=dict(required=True, type='bool'),
        save_config=dict(default=False, type='bool')
    )

    argument_spec.update(ce_argument_spec)
    module = Reboot(argument_spec=argument_spec, supports_check_mode=True)

    if not HAS_NCCLIENT:
        module.network_module.fail_json(msg='Error: The ncclient library is required.')

    changed = False
    rebooted = False

    module.work()

    changed = True
    rebooted = True

    results = dict()
    results['changed'] = changed
    results['rebooted'] = rebooted

    module.network_module.exit_json(**results)


if __name__ == '__main__':
    main()



from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import pytest

openshiftdynamic = pytest.importorskip("openshift.dynamic")

from ansible_collections.community.kubevirt.tests.unit.plugins.modules.utils import set_module_args
from .kubevirt_fixtures import base_fixture, RESOURCE_DEFAULT_ARGS, AnsibleExitJson

from ansible_collections.community.kubernetes.plugins.module_utils.raw import KubernetesRawModule
from ansible_collections.community.kubevirt.plugins.modules import kubevirt_rs as mymodule

KIND = 'VirtualMachineInstanceReplicaSet'


@pytest.mark.usefixtures("base_fixture")
@pytest.mark.parametrize("_replicas, _changed", ((1, True),
                                                 (3, True),
                                                 (2, False),
                                                 (5, True),))
def test_scale_rs_nowait(_replicas, _changed):
    _name = 'test-rs'
    # Desired state:
    args = dict(name=_name, namespace='vms', replicas=_replicas, wait=False)
    set_module_args(args)

    # Mock pre-change state:
    resource_args = dict(kind=KIND, **RESOURCE_DEFAULT_ARGS)
    mymodule.KubeVirtVMIRS.find_supported_resource.return_value = openshiftdynamic.Resource(**resource_args)
    res_inst = openshiftdynamic.ResourceInstance('', dict(kind=KIND, metadata={'name': _name}, spec={'replicas': 2}))
    openshiftdynamic.Resource.get.return_value = res_inst
    openshiftdynamic.Resource.search.return_value = [res_inst]

    # Final state, after patching the object
    KubernetesRawModule.patch_resource.return_value = dict(kind=KIND, metadata={'name': _name},
                                                           spec={'replicas': _replicas}), None

    # Run code:
    with pytest.raises(AnsibleExitJson) as result:
        mymodule.KubeVirtVMIRS().execute_module()

    # Verify result:
    assert result.value['changed'] == _changed


@pytest.mark.usefixtures("base_fixture")
@pytest.mark.parametrize("_replicas, _success", ((1, False),
                                                 (2, False),
                                                 (5, True),))
def test_scale_rs_wait(_replicas, _success):
    _name = 'test-rs'
    # Desired state:
    args = dict(name=_name, namespace='vms', replicas=5, wait=True)
    set_module_args(args)

    # Mock pre-change state:
    resource_args = dict(kind=KIND, **RESOURCE_DEFAULT_ARGS)
    mymodule.KubeVirtVMIRS.find_supported_resource.return_value = openshiftdynamic.Resource(**resource_args)
    res_inst = openshiftdynamic.ResourceInstance('', dict(kind=KIND, metadata={'name': _name}, spec={'replicas': 2}))
    openshiftdynamic.Resource.get.return_value = res_inst
    openshiftdynamic.Resource.search.return_value = [res_inst]

    # ~Final state, after patching the object (`replicas` match desired state)
    KubernetesRawModule.patch_resource.return_value = dict(kind=KIND, name=_name, metadata={'name': _name},
                                                           spec={'replicas': 5}), None

    # Final final state, as returned by resource.watch()
    final_obj = dict(metadata=dict(name=_name), status=dict(readyReplicas=_replicas), **resource_args)
    event = openshiftdynamic.ResourceInstance(None, final_obj)
    openshiftdynamic.Resource.watch.return_value = [dict(object=event)]

    # Run code:
    with pytest.raises(Exception) as result:
        mymodule.KubeVirtVMIRS().execute_module()

    # Verify result:
    assert result.value['success'] == _success



#!/usr/bin/env python

"""
Spacewalk external inventory script
=================================

Ansible has a feature where instead of reading from /etc/ansible/hosts
as a text file, it can query external programs to obtain the list
of hosts, groups the hosts are in, and even variables to assign to each host.

To use this, copy this file over /etc/ansible/hosts and chmod +x the file.
This, more or less, allows you to keep one central database containing
info about all of your managed instances.

This script is dependent upon the spacealk-reports package being installed
on the same machine. It is basically a CSV-to-JSON converter from the
output of "spacewalk-report system-groups-systems|inventory".

Tested with Ansible 1.9.2 and spacewalk 2.3
"""
#
# Author:: Jon Miller <jonEbird@gmail.com>
# Copyright:: Copyright (c) 2013, Jon Miller
#
# Extended for support of multiple organizations and
# adding the "_meta" dictionary to --list output by
# Bernhard Lichtinger <bernhard.lichtinger@lrz.de> 2015
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import sys
import os
import time
from optparse import OptionParser
import subprocess
import json

from ansible.module_utils.six import iteritems
from ansible.module_utils.six.moves import configparser as ConfigParser


base_dir = os.path.dirname(os.path.realpath(__file__))
default_ini_file = os.path.join(base_dir, "spacewalk.ini")

SW_REPORT = '/usr/bin/spacewalk-report'
CACHE_DIR = os.path.join(base_dir, ".spacewalk_reports")
CACHE_AGE = 300  # 5min
INI_FILE = os.path.expanduser(os.path.expandvars(os.environ.get("SPACEWALK_INI_PATH", default_ini_file)))


# Sanity check
if not os.path.exists(SW_REPORT):
    print('Error: %s is required for operation.' % (SW_REPORT), file=sys.stderr)
    sys.exit(1)

# Pre-startup work
if not os.path.exists(CACHE_DIR):
    os.mkdir(CACHE_DIR)
    os.chmod(CACHE_DIR, 0o2775)

# Helper functions
# ------------------------------


def spacewalk_report(name):
    """Yield a dictionary form of each CSV output produced by the specified
    spacewalk-report
    """
    cache_filename = os.path.join(CACHE_DIR, name)
    if not os.path.exists(cache_filename) or \
            (time.time() - os.stat(cache_filename).st_mtime) > CACHE_AGE:
        # Update the cache
        fh = open(cache_filename, 'w')
        p = subprocess.Popen([SW_REPORT, name], stdout=fh)
        p.wait()
        fh.close()

    with open(cache_filename, 'r') as f:
        lines = f.readlines()
    keys = lines[0].strip().split(',')
    # add 'spacewalk_' prefix to the keys
    keys = ['spacewalk_' + key for key in keys]
    for line in lines[1:]:
        values = line.strip().split(',')
        if len(keys) == len(values):
            yield dict(zip(keys, values))


# Options
# ------------------------------

parser = OptionParser(usage="%prog [options] --list | --host <machine>")
parser.add_option('--list', default=False, dest="list", action="store_true",
                  help="Produce a JSON consumable grouping of servers for Ansible")
parser.add_option('--host', default=None, dest="host",
                  help="Generate additional host specific details for given host for Ansible")
parser.add_option('-H', '--human', dest="human",
                  default=False, action="store_true",
                  help="Produce a friendlier version of either server list or host detail")
parser.add_option('-o', '--org', default=None, dest="org_number",
                  help="Limit to spacewalk organization number")
parser.add_option('-p', default=False, dest="prefix_org_name", action="store_true",
                  help="Prefix the group name with the organization number")
(options, args) = parser.parse_args()


# read spacewalk.ini if present
# ------------------------------
if os.path.exists(INI_FILE):
    config = ConfigParser.SafeConfigParser()
    config.read(INI_FILE)
    if config.has_option('spacewalk', 'cache_age'):
        CACHE_AGE = config.get('spacewalk', 'cache_age')
    if not options.org_number and config.has_option('spacewalk', 'org_number'):
        options.org_number = config.get('spacewalk', 'org_number')
    if not options.prefix_org_name and config.has_option('spacewalk', 'prefix_org_name'):
        options.prefix_org_name = config.getboolean('spacewalk', 'prefix_org_name')


# Generate dictionary for mapping group_id to org_id
# ------------------------------
org_groups = {}
try:
    for group in spacewalk_report('system-groups'):
        org_groups[group['spacewalk_group_id']] = group['spacewalk_org_id']

except (OSError) as e:
    print('Problem executing the command "%s system-groups": %s' %
          (SW_REPORT, str(e)), file=sys.stderr)
    sys.exit(2)


# List out the known server from Spacewalk
# ------------------------------
if options.list:

    # to build the "_meta"-Group with hostvars first create dictionary for later use
    host_vars = {}
    try:
        for item in spacewalk_report('inventory'):
            host_vars[item['spacewalk_profile_name']] = dict((key, (value.split(';') if ';' in value else value)) for key, value in item.items())

    except (OSError) as e:
        print('Problem executing the command "%s inventory": %s' %
              (SW_REPORT, str(e)), file=sys.stderr)
        sys.exit(2)

    groups = {}
    meta = {"hostvars": {}}
    try:
        for system in spacewalk_report('system-groups-systems'):
            # first get org_id of system
            org_id = org_groups[system['spacewalk_group_id']]

            # shall we add the org_id as prefix to the group name:
            if options.prefix_org_name:
                prefix = org_id + "-"
                group_name = prefix + system['spacewalk_group_name']
            else:
                group_name = system['spacewalk_group_name']

            # if we are limited to one organization:
            if options.org_number:
                if org_id == options.org_number:
                    if group_name not in groups:
                        groups[group_name] = set()

                    groups[group_name].add(system['spacewalk_server_name'])
                    if system['spacewalk_server_name'] in host_vars and not system['spacewalk_server_name'] in meta["hostvars"]:
                        meta["hostvars"][system['spacewalk_server_name']] = host_vars[system['spacewalk_server_name']]
            # or we list all groups and systems:
            else:
                if group_name not in groups:
                    groups[group_name] = set()

                groups[group_name].add(system['spacewalk_server_name'])
                if system['spacewalk_server_name'] in host_vars and not system['spacewalk_server_name'] in meta["hostvars"]:
                    meta["hostvars"][system['spacewalk_server_name']] = host_vars[system['spacewalk_server_name']]

    except (OSError) as e:
        print('Problem executing the command "%s system-groups-systems": %s' %
              (SW_REPORT, str(e)), file=sys.stderr)
        sys.exit(2)

    if options.human:
        for group, systems in iteritems(groups):
            print('[%s]\n%s\n' % (group, '\n'.join(systems)))
    else:
        final = dict([(k, list(s)) for k, s in iteritems(groups)])
        final["_meta"] = meta
        print(json.dumps(final))
        # print(json.dumps(groups))
    sys.exit(0)


# Return a details information concerning the spacewalk server
# ------------------------------
elif options.host:

    host_details = {}
    try:
        for system in spacewalk_report('inventory'):
            if system['spacewalk_hostname'] == options.host:
                host_details = system
                break

    except (OSError) as e:
        print('Problem executing the command "%s inventory": %s' %
              (SW_REPORT, str(e)), file=sys.stderr)
        sys.exit(2)

    if options.human:
        print('Host: %s' % options.host)
        for k, v in iteritems(host_details):
            print('  %s: %s' % (k, '\n    '.join(v.split(';'))))
    else:
        print(json.dumps(dict((key, (value.split(';') if ';' in value else value)) for key, value in host_details.items())))
    sys.exit(0)

else:

    parser.print_help()
    sys.exit(1)



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The ios_static_routes fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type


from copy import deepcopy
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import (
    utils,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.utils.utils import (
    netmask_to_cidr,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.static_routes.static_routes import (
    Static_RoutesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.utils.utils import (
    is_valid_ip,
)


class Static_RoutesFacts(object):
    """ The ios_static_routes fact class
    """

    def __init__(self, module, subspec="config", options="options"):

        self._module = module
        self.argument_spec = Static_RoutesArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_static_routes_data(self, connection):
        return connection.get(
            "sh running-config | section ^ip route|ipv6 route"
        )

    def populate_facts(self, connection, ansible_facts, data=None):
        """ Populate the facts for static_routes
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """

        objs = []
        if not data:
            data = self.get_static_routes_data(connection)
        # operate on a collection of resource x
        config = data.split("\n")

        same_dest = self.populate_destination(config)
        for key in same_dest.keys():
            if key:
                obj = self.render_config(
                    self.generated_spec, key, same_dest[key]
                )
                if obj:
                    objs.append(obj)
        facts = {}

        # append all static routes address_family with NO VRF together
        no_vrf_address_family = {
            "address_families": [
                each.get("address_families")[0]
                for each in objs
                if each.get("vrf") is None
            ]
        }

        temp_objs = [each for each in objs if each.get("vrf") is not None]
        temp_objs.append(no_vrf_address_family)
        objs = temp_objs

        if objs:
            facts["static_routes"] = []
            params = utils.validate_config(
                self.argument_spec, {"config": objs}
            )
            for cfg in params["config"]:
                facts["static_routes"].append(utils.remove_empties(cfg))
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts

    def update_netmask_to_cidr(self, filter, pos, del_pos):
        netmask = filter.split(" ")
        dest = netmask[pos] + "/" + netmask_to_cidr(netmask[del_pos])
        netmask[pos] = dest
        del netmask[del_pos]
        filter_vrf = " "
        return filter_vrf.join(netmask), dest

    def populate_destination(self, config):
        same_dest = {}
        ip_str = ""
        for i in sorted(config):
            if i and "ospf" not in i:
                if "::" in i and "vrf" in i:
                    ip_str = "ipv6 route vrf"
                elif "::" in i and "vrf" not in i:
                    ip_str = "ipv6 route"
                elif "." in i and "vrf" in i:
                    ip_str = "ip route vrf"
                elif "." in i and "vrf" not in i:
                    ip_str = "ip route"

                if "vrf" in i:
                    filter_vrf = utils.parse_conf_arg(i, ip_str)
                    if "::" not in filter_vrf:
                        filter_vrf, dest_vrf = self.update_netmask_to_cidr(
                            filter_vrf, 1, 2
                        )
                        dest_vrf = dest_vrf + "_vrf"
                    else:
                        dest_vrf = filter_vrf.split(" ")[1]
                    if dest_vrf not in same_dest.keys():
                        same_dest[dest_vrf] = []
                        same_dest[dest_vrf].append("vrf " + filter_vrf)
                    elif "vrf" not in same_dest[dest_vrf][0]:
                        same_dest[dest_vrf] = []
                        same_dest[dest_vrf].append("vrf " + filter_vrf)
                    else:
                        same_dest[dest_vrf].append(("vrf " + filter_vrf))
                else:
                    filter_non_vrf = utils.parse_conf_arg(i, ip_str)
                    if (
                        "::" not in filter_non_vrf
                    ):  # "/" not in filter_non_vrf and
                        filter_non_vrf, dest = self.update_netmask_to_cidr(
                            filter_non_vrf, 0, 1
                        )
                    else:
                        dest = filter_non_vrf.split(" ")[0]
                    if dest not in same_dest.keys():
                        same_dest[dest] = []
                        same_dest[dest].append(filter_non_vrf)
                    elif "vrf" in same_dest[dest][0]:
                        same_dest[dest] = []
                        same_dest[dest].append(filter_non_vrf)
                    else:
                        same_dest[dest].append(filter_non_vrf)
        return same_dest

    def render_config(self, spec, conf, conf_val):
        """
        Render config as dictionary structure and delete keys
          from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)
        config["address_families"] = []
        route_dict = dict()
        final_route = dict()
        afi = dict()
        final_route["routes"] = []
        next_hops = []
        hops = {}
        vrf = ""
        address_family = dict()
        for each in conf_val:
            route = each.split(" ")
            if "vrf" in conf_val[0]:
                vrf = route[route.index("vrf") + 1]
                route_dict["dest"] = conf.split("_")[0]
            else:
                route_dict["dest"] = conf
            if "vrf" in conf_val[0]:
                hops = {}
                if "::" in conf:
                    hops["forward_router_address"] = route[3]
                    afi["afi"] = "ipv6"
                elif "." in conf:
                    if is_valid_ip(route[3]):
                        hops["forward_router_address"] = route[3]
                        afi["afi"] = "ipv4"
                    else:
                        hops["interface"] = route[3]
                        afi["afi"] = "ipv4"
                        if is_valid_ip(route[4]):
                            hops["forward_router_address"] = route[4]
            else:

                if "::" in conf:
                    if is_valid_ip(route[1]):
                        hops["forward_router_address"] = route[1]
                        afi["afi"] = "ipv6"
                    else:
                        hops["interface"] = route[1]
                        afi["afi"] = "ipv6"
                        if is_valid_ip(route[2]):
                            hops["forward_router_address"] = route[2]
                elif "." in conf:
                    if is_valid_ip(route[1]):
                        hops["forward_router_address"] = route[1]
                        afi["afi"] = "ipv4"
                    else:
                        hops["interface"] = route[1]
                        afi["afi"] = "ipv4"
                        if is_valid_ip(route[2]):
                            hops["forward_router_address"] = route[2]
            try:
                temp_list = each.split(" ")
                if "tag" in temp_list:
                    del temp_list[temp_list.index("tag") + 1]
                if "track" in temp_list:
                    del temp_list[temp_list.index("track") + 1]
                # find distance metric
                dist_metrics = int(
                    [
                        i
                        for i in temp_list
                        if "." not in i
                        and ":" not in i
                        and ord(i[0]) > 48
                        and ord(i[0]) < 57
                    ][0]
                )
            except IndexError:
                dist_metrics = None
            if dist_metrics:
                hops["distance_metric"] = dist_metrics
            if "name" in route:
                hops["name"] = route[route.index("name") + 1]
            if "multicast" in route:
                hops["multicast"] = True
            if "dhcp" in route:
                hops["dhcp"] = True
            if "global" in route:
                hops["global"] = True
            if "permanent" in route:
                hops["permanent"] = True
            if "tag" in route:
                hops["tag"] = route[route.index("tag") + 1]
            if "track" in route:
                hops["track"] = route[route.index("track") + 1]
            next_hops.append(hops)
            hops = {}
        route_dict["next_hops"] = next_hops
        if route_dict:
            final_route["routes"].append(route_dict)
        address_family.update(afi)
        address_family.update(final_route)
        config["address_families"].append(address_family)
        if vrf:
            config["vrf"] = vrf

        return utils.remove_empties(config)



#
# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.cisco.nxos.plugins.modules import nxos_lldp_interfaces
from ansible_collections.cisco.nxos.tests.unit.compat.mock import (
    patch,
    MagicMock,
)
from ansible_collections.cisco.nxos.tests.unit.modules.utils import (
    set_module_args,
)
from .nxos_module import TestNxosModule, load_fixture


class TestNxosLldpInterfacesModule(TestNxosModule):

    module = nxos_lldp_interfaces

    def setUp(self):
        super(TestNxosLldpInterfacesModule, self).setUp()

        self.mock_get_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.get_config"
        )
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_get_resource_connection_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base.get_resource_connection"
        )
        self.get_resource_connection_config = (
            self.mock_get_resource_connection_config.start()
        )

        self.mock_get_resource_connection_facts = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.facts.facts.get_resource_connection"
        )
        self.get_resource_connection_facts = (
            self.mock_get_resource_connection_facts.start()
        )

        self.mock_edit_config = patch(
            "ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.lldp_interfaces.lldp_interfaces.Lldp_interfaces.edit_config"
        )
        self.edit_config = self.mock_edit_config.start()

        self.mock_execute_show_command = patch(
            "ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.lldp_interfaces.lldp_interfaces.Lldp_interfacesFacts.get_device_data"
        )
        self.execute_show_command = self.mock_execute_show_command.start()

    def tearDown(self):
        super(TestNxosLldpInterfacesModule, self).tearDown()
        self.mock_get_resource_connection_config.stop()
        self.mock_get_resource_connection_facts.stop()
        self.mock_edit_config.stop()
        self.mock_get_config.stop()
        self.mock_load_config.stop()
        self.mock_execute_show_command.stop()

    def load_fixtures(self, commands=None, device=""):
        def load_from_file(*args, **kwargs):
            output = """interface Ethernet1/1
            lldp receive
            no lldp transmit
          interface Ethernet1/2
            no lldp receive
            lldp tlv-set vlan 12"""
            return output

        self.execute_show_command.side_effect = load_from_file

    def test_nxos_lldp_interfaces_merged(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        name="Ethernet1/3",
                        receive=False,
                        tlv_set=dict(vlan=123),
                    )
                ],
                state="merged",
            )
        )
        commands = [
            "interface Ethernet1/3",
            "no lldp receive",
            "lldp tlv-set vlan 123",
        ]
        self.execute_module(changed=True, commands=commands)

    def test_nxos_lldp_interfaces_merged_idempotent(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        name="Ethernet1/2",
                        receive=False,
                        tlv_set=dict(vlan=12),
                    ),
                    dict(name="Ethernet1/1", receive=True, transmit=False),
                ],
                state="merged",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_nxos_lldp_interfaces_replaced(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        name="Ethernet1/2",
                        receive=True,
                        transmit=False,
                        tlv_set=dict(management_address="192.0.2.123"),
                    )
                ],
                state="replaced",
            )
        )
        commands = [
            "interface Ethernet1/2",
            "lldp receive",
            "no lldp transmit",
            "no lldp tlv-set vlan 12",
            "lldp tlv-set management-address 192.0.2.123",
        ]
        self.execute_module(changed=True, commands=commands)

    def test_nxos_lldp_interfaces_replaced_idempotent(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        name="Ethernet1/2",
                        receive=False,
                        tlv_set=dict(vlan=12),
                    ),
                    dict(name="Ethernet1/1", receive=True, transmit=False),
                ],
                state="replaced",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_nxos_lldp_interfaces_overridden(self):
        set_module_args(
            dict(
                config=[
                    dict(name="Ethernet1/4", receive=True, transmit=False)
                ],
                state="overridden",
            )
        )
        commands = [
            "interface Ethernet1/4",
            "lldp receive",
            "no lldp transmit",
            "interface Ethernet1/1",
            "lldp receive",
            "lldp transmit",
            "interface Ethernet1/2",
            "lldp receive",
            "no lldp tlv-set vlan 12",
        ]
        self.execute_module(changed=True, commands=commands)

    def test_nxos_lldp_interfaces_overridden_idempotent(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        name="Ethernet1/2",
                        receive=False,
                        tlv_set=dict(vlan=12),
                    ),
                    dict(name="Ethernet1/1", receive=True, transmit=False),
                ],
                state="overridden",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_nxos_lldp_interfaces_deleted_intf(self):
        set_module_args(
            dict(config=[dict(name="Ethernet1/2")], state="deleted")
        )
        commands = [
            "interface Ethernet1/2",
            "lldp receive",
            "no lldp tlv-set vlan 12",
        ]
        self.execute_module(changed=True, commands=commands)

    def test_nxos_lldp_interfaces_deleted_all(self):
        set_module_args(dict(state="deleted"))
        commands = [
            "interface Ethernet1/2",
            "lldp receive",
            "no lldp tlv-set vlan 12",
            "interface Ethernet1/1",
            "lldp receive",
            "lldp transmit",
        ]
        self.execute_module(changed=True, commands=commands)

    def test_nxos_lldp_interfaces_rendered(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        name="Ethernet1/2",
                        receive=False,
                        tlv_set=dict(vlan=12),
                    ),
                    dict(name="Ethernet1/1", receive=True, transmit=False),
                ],
                state="rendered",
            )
        )
        commands = [
            "interface Ethernet1/1",
            "lldp receive",
            "no lldp transmit",
            "interface Ethernet1/2",
            "no lldp receive",
            "lldp tlv-set vlan 12",
        ]
        result = self.execute_module(changed=False)
        self.assertEqual(
            sorted(result["rendered"]), sorted(commands), result["rendered"]
        )

    def test_nxos_lldp_interfaces_parsed(self):
        set_module_args(
            dict(
                running_config="""interface Ethernet1/1
            lldp receive
            no lldp transmit
          interface Ethernet1/2
            no lldp receive
            lldp tlv-set vlan 12""",
                state="parsed",
            )
        )
        result = self.execute_module(changed=False)
        compare_list = [
            {"name": "Ethernet1/1", "receive": True, "transmit": False},
            {"name": "Ethernet1/2", "receive": False, "tlv_set": {"vlan": 12}},
        ]
        self.assertEqual(result["parsed"], compare_list, result["parsed"])

    def test_nxos_lldp_interfaces_gathered(self):
        set_module_args(dict(state="gathered"))
        result = self.execute_module(changed=False)
        compare_list = [
            {"name": "Ethernet1/1", "receive": True, "transmit": False},
            {"name": "Ethernet1/2", "receive": False, "tlv_set": {"vlan": 12}},
        ]
        self.assertEqual(result["gathered"], compare_list, result["gathered"])



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
# pylint: skip-file
from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: nxos_nxapi
extends_documentation_fragment:
- cisco.nxos.nxos
author: Peter Sprygada (@privateip)
short_description: Manage NXAPI configuration on an NXOS device.
description:
- Configures the NXAPI feature on devices running Cisco NXOS.  The NXAPI feature is
  absent from the configuration by default.  Since this module manages the NXAPI feature
  it only supports the use of the C(Cli) transport.
version_added: 1.0.0
options:
  http_port:
    description:
    - Configure the port with which the HTTP server will listen on for requests.  By
      default, NXAPI will bind the HTTP service to the standard HTTP port 80.  This
      argument accepts valid port values in the range of 1 to 65535.
    required: false
    default: 80
    type: int
  http:
    description:
    - Controls the operating state of the HTTP protocol as one of the underlying transports
      for NXAPI.  By default, NXAPI will enable the HTTP transport when the feature
      is first configured.  To disable the use of the HTTP transport, set the value
      of this argument to False.
    required: false
    default: true
    type: bool
    aliases:
    - enable_http
  https_port:
    description:
    - Configure the port with which the HTTPS server will listen on for requests.  By
      default, NXAPI will bind the HTTPS service to the standard HTTPS port 443.  This
      argument accepts valid port values in the range of 1 to 65535.
    required: false
    default: 443
    type: int
  https:
    description:
    - Controls the operating state of the HTTPS protocol as one of the underlying
      transports for NXAPI.  By default, NXAPI will disable the HTTPS transport when
      the feature is first configured.  To enable the use of the HTTPS transport,
      set the value of this argument to True.
    required: false
    default: false
    type: bool
    aliases:
    - enable_https
  sandbox:
    description:
    - The NXAPI feature provides a web base UI for developers for entering commands.  This
      feature is initially disabled when the NXAPI feature is configured for the first
      time.  When the C(sandbox) argument is set to True, the developer sandbox URL
      will accept requests and when the value is set to False, the sandbox URL is
      unavailable. This is supported on NX-OS 7K series.
    required: false
    type: bool
    aliases:
    - enable_sandbox
  state:
    description:
    - The C(state) argument controls whether or not the NXAPI feature is configured
      on the remote device.  When the value is C(present) the NXAPI feature configuration
      is present in the device running-config.  When the values is C(absent) the feature
      configuration is removed from the running-config.
    choices:
    - present
    - absent
    required: false
    default: present
    type: str
  ssl_strong_ciphers:
    description:
    - Controls the use of whether strong or weak ciphers are configured. By default,
      this feature is disabled and weak ciphers are configured.  To enable the use
      of strong ciphers, set the value of this argument to True.
    required: false
    default: false
    type: bool
  tlsv1_0:
    description:
    - Controls the use of the Transport Layer Security version 1.0 is configured.  By
      default, this feature is enabled.  To disable the use of TLSV1.0, set the value
      of this argument to True.
    required: false
    default: true
    type: bool
  tlsv1_1:
    description:
    - Controls the use of the Transport Layer Security version 1.1 is configured.  By
      default, this feature is disabled.  To enable the use of TLSV1.1, set the value
      of this argument to True.
    required: false
    default: false
    type: bool
  tlsv1_2:
    description:
    - Controls the use of the Transport Layer Security version 1.2 is configured.  By
      default, this feature is disabled.  To enable the use of TLSV1.2, set the value
      of this argument to True.
    required: false
    default: false
    type: bool
"""

EXAMPLES = """
- name: Enable NXAPI access with default configuration
  cisco.nxos.nxos_nxapi:
    state: present

- name: Enable NXAPI with no HTTP, HTTPS at port 9443 and sandbox disabled
  cisco.nxos.nxos_nxapi:
    enable_http: false
    https_port: 9443
    https: yes
    enable_sandbox: no

- name: remove NXAPI configuration
  cisco.nxos.nxos_nxapi:
    state: absent
"""

RETURN = """
updates:
  description:
    - Returns the list of commands that need to be pushed into the remote
      device to satisfy the arguments
  returned: always
  type: list
  sample: ['no feature nxapi']
"""
import re

from distutils.version import LooseVersion
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    run_commands,
    load_config,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    nxos_argument_spec,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_capabilities,
)
from ansible.module_utils.basic import AnsibleModule


def check_args(module, warnings, capabilities):
    network_api = capabilities.get("network_api", "nxapi")
    if network_api == "nxapi":
        module.fail_json(msg="module not supported over nxapi transport")

    os_platform = capabilities["device_info"]["network_os_platform"]
    if "7K" not in os_platform and module.params["sandbox"]:
        module.fail_json(
            msg="sandbox or enable_sandbox is supported on NX-OS 7K series of switches"
        )

    state = module.params["state"]

    if state == "started":
        module.params["state"] = "present"
        warnings.append(
            "state=started is deprecated and will be removed in a "
            "a future release.  Please use state=present instead"
        )
    elif state == "stopped":
        module.params["state"] = "absent"
        warnings.append(
            "state=stopped is deprecated and will be removed in a "
            "a future release.  Please use state=absent instead"
        )

    for key in ["http_port", "https_port"]:
        if module.params[key] is not None:
            if not 1 <= module.params[key] <= 65535:
                module.fail_json(msg="%s must be between 1 and 65535" % key)

    return warnings


def map_obj_to_commands(want, have, module, warnings, capabilities):
    send_commands = list()
    commands = dict()
    os_platform = None
    os_version = None

    device_info = capabilities.get("device_info")
    if device_info:
        os_version = device_info.get("network_os_version")
        if os_version:
            os_version = os_version[:3]
        os_platform = device_info.get("network_os_platform")
        if os_platform:
            os_platform = os_platform[:3]

    def needs_update(x):
        return want.get(x) is not None and (want.get(x) != have.get(x))

    if needs_update("state"):
        if want["state"] == "absent":
            return ["no feature nxapi"]
        send_commands.append("feature nxapi")
    elif want["state"] == "absent":
        return send_commands

    for parameter in ["http", "https"]:
        port_param = parameter + "_port"
        if needs_update(parameter):
            if want.get(parameter) is False:
                commands[parameter] = "no nxapi %s" % parameter
            else:
                commands[parameter] = "nxapi %s port %s" % (
                    parameter,
                    want.get(port_param),
                )

        if needs_update(port_param) and want.get(parameter) is True:
            commands[parameter] = "nxapi %s port %s" % (
                parameter,
                want.get(port_param),
            )

    if needs_update("sandbox"):
        commands["sandbox"] = "nxapi sandbox"
        if not want["sandbox"]:
            commands["sandbox"] = "no %s" % commands["sandbox"]

    if os_platform and os_version:
        if (os_platform == "N9K" or os_platform == "N3K") and LooseVersion(
            os_version
        ) >= "9.2":
            if needs_update("ssl_strong_ciphers"):
                commands["ssl_strong_ciphers"] = "nxapi ssl ciphers weak"
                if want["ssl_strong_ciphers"] is True:
                    commands[
                        "ssl_strong_ciphers"
                    ] = "no nxapi ssl ciphers weak"

            have_ssl_protocols = ""
            want_ssl_protocols = ""
            for key, value in {
                "tlsv1_2": "TLSv1.2",
                "tlsv1_1": "TLSv1.1",
                "tlsv1_0": "TLSv1",
            }.items():
                if needs_update(key):
                    if want.get(key) is True:
                        want_ssl_protocols = " ".join(
                            [want_ssl_protocols, value]
                        )
                elif have.get(key) is True:
                    have_ssl_protocols = " ".join([have_ssl_protocols, value])

            if len(want_ssl_protocols) > 0:
                commands["ssl_protocols"] = "nxapi ssl protocols%s" % (
                    " ".join([want_ssl_protocols, have_ssl_protocols])
                )
    else:
        warnings.append(
            "os_version and/or os_platform keys from "
            "platform capabilities are not available.  "
            "Any NXAPI SSL optional arguments will be ignored"
        )

    send_commands.extend(commands.values())

    return send_commands


def parse_http(data):
    http_res = [r"nxapi http port (\d+)"]
    http_port = None

    for regex in http_res:
        match = re.search(regex, data, re.M)
        if match:
            http_port = int(match.group(1))
            break

    return {"http": http_port is not None, "http_port": http_port}


def parse_https(data):
    https_res = [r"nxapi https port (\d+)"]
    https_port = None

    for regex in https_res:
        match = re.search(regex, data, re.M)
        if match:
            https_port = int(match.group(1))
            break

    return {"https": https_port is not None, "https_port": https_port}


def parse_sandbox(data):
    sandbox = [
        item for item in data.split("\n") if re.search(r".*sandbox.*", item)
    ]
    value = False
    if sandbox and sandbox[0] == "nxapi sandbox":
        value = True
    return {"sandbox": value}


def parse_ssl_strong_ciphers(data):
    ciphers_res = [r"(\w+) nxapi ssl ciphers weak"]
    value = None

    for regex in ciphers_res:
        match = re.search(regex, data, re.M)
        if match:
            value = match.group(1)
            break

    return {"ssl_strong_ciphers": value == "no"}


def parse_ssl_protocols(data):
    tlsv1_0 = re.search(r"(?<!\S)TLSv1(?!\S)", data, re.M) is not None
    tlsv1_1 = re.search(r"(?<!\S)TLSv1.1(?!\S)", data, re.M) is not None
    tlsv1_2 = re.search(r"(?<!\S)TLSv1.2(?!\S)", data, re.M) is not None

    return {"tlsv1_0": tlsv1_0, "tlsv1_1": tlsv1_1, "tlsv1_2": tlsv1_2}


def map_config_to_obj(module):
    out = run_commands(module, ["show run all | inc nxapi"], check_rc=False)[0]
    match = re.search(r"no feature nxapi", out, re.M)
    # There are two possible outcomes when nxapi is disabled on nxos platforms.
    # 1. Nothing is displayed in the running config.
    # 2. The 'no feature nxapi' command is displayed in the running config.
    if match or out == "":
        return {"state": "absent"}

    out = str(out).strip()

    obj = {"state": "present"}
    obj.update(parse_http(out))
    obj.update(parse_https(out))
    obj.update(parse_sandbox(out))
    obj.update(parse_ssl_strong_ciphers(out))
    obj.update(parse_ssl_protocols(out))

    return obj


def map_params_to_obj(module):
    obj = {
        "http": module.params["http"],
        "http_port": module.params["http_port"],
        "https": module.params["https"],
        "https_port": module.params["https_port"],
        "sandbox": module.params["sandbox"],
        "state": module.params["state"],
        "ssl_strong_ciphers": module.params["ssl_strong_ciphers"],
        "tlsv1_0": module.params["tlsv1_0"],
        "tlsv1_1": module.params["tlsv1_1"],
        "tlsv1_2": module.params["tlsv1_2"],
    }

    return obj


def main():
    """ main entry point for module execution
    """
    argument_spec = dict(
        http=dict(aliases=["enable_http"], type="bool", default=True),
        http_port=dict(type="int", default=80),
        https=dict(aliases=["enable_https"], type="bool", default=False),
        https_port=dict(type="int", default=443),
        sandbox=dict(aliases=["enable_sandbox"], type="bool"),
        state=dict(default="present", choices=["present", "absent"]),
        ssl_strong_ciphers=dict(type="bool", default=False),
        tlsv1_0=dict(type="bool", default=True),
        tlsv1_1=dict(type="bool", default=False),
        tlsv1_2=dict(type="bool", default=False),
    )

    argument_spec.update(nxos_argument_spec)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )

    warnings = list()
    warning_msg = (
        "Module nxos_nxapi currently defaults to configure 'http port 80'. "
    )
    warning_msg += "Default behavior is changing to configure 'https port 443'"
    warning_msg += " when params 'http, http_port, https, https_port' are not set in the playbook"
    module.deprecate(
        msg=warning_msg, date="2022-06-01", collection_name="cisco.nxos"
    )

    capabilities = get_capabilities(module)

    check_args(module, warnings, capabilities)

    want = map_params_to_obj(module)
    have = map_config_to_obj(module)

    commands = map_obj_to_commands(want, have, module, warnings, capabilities)

    result = {"changed": False, "warnings": warnings, "commands": commands}

    if commands:
        if not module.check_mode:
            load_config(module, commands)
        result["changed"] = True

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: aci_epg
short_description: Manage End Point Groups (EPG) objects (fv:AEPg)
description:
- Manage End Point Groups (EPG) on Cisco ACI fabrics.
options:
  tenant:
    description:
    - Name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  ap:
    description:
    - Name of an existing application network profile, that will contain the EPGs.
    type: str
    aliases: [ app_profile, app_profile_name ]
  epg:
    description:
    - Name of the end point group.
    type: str
    aliases: [ epg_name, name ]
  bd:
    description:
    - Name of the bridge domain being associated with the EPG.
    type: str
    aliases: [ bd_name, bridge_domain ]
  priority:
    description:
    - The QoS class.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ level1, level2, level3, unspecified ]
  intra_epg_isolation:
    description:
    - The Intra EPG Isolation.
    - The APIC defaults to C(unenforced) when unset during creation.
    type: str
    choices: [ enforced, unenforced ]
  description:
    description:
    - Description for the EPG.
    type: str
    aliases: [ descr ]
  fwd_control:
    description:
    - The forwarding control used by the EPG.
    - The APIC defaults to C(none) when unset during creation.
    type: str
    choices: [ none, proxy-arp ]
  preferred_group:
    description:
    - Whether ot not the EPG is part of the Preferred Group and can communicate without contracts.
    - This is very convenient for migration scenarios, or when ACI is used for network automation but not for policy.
    - The APIC defaults to C(no) when unset during creation.
    type: bool
  monitoring_policy:
    description:
    - The name of the monitoring policy.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci

notes:
- The C(tenant) and C(app_profile) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) and M(cisco.aci.aci_ap) modules can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_ap
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fv:AEPg).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Swetha Chunduri (@schunduri)
- Shreyas Srish (@shrsr)
'''

EXAMPLES = r'''
- name: Add a new EPG
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    ap: intranet
    epg: web_epg
    description: Web Intranet EPG
    bd: prod_bd
    monitoring_policy: default
    preferred_group: yes
    state: present
  delegate_to: localhost

- aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    ap: ticketing
    epg: "{{ item.epg }}"
    description: Ticketing EPG
    bd: "{{ item.bd }}"
    priority: unspecified
    intra_epg_isolation: unenforced
    state: present
  delegate_to: localhost
  with_items:
    - epg: web
      bd: web_bd
    - epg: database
      bd: database_bd

- name: Remove an EPG
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    validate_certs: no
    tenant: production
    app_profile: intranet
    epg: web_epg
    monitoring_policy: default
    state: absent
  delegate_to: localhost

- name: Query an EPG
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    ap: ticketing
    epg: web_epg
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all EPGs
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all EPGs with a Specific Name
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    validate_certs: no
    epg: web_epg
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all EPGs of an App Profile
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    validate_certs: no
    ap: ticketing
    state: query
  delegate_to: localhost
  register: query_result
'''

RETURN = r'''
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(
        epg=dict(type='str', aliases=['epg_name', 'name']),  # Not required for querying all objects
        bd=dict(type='str', aliases=['bd_name', 'bridge_domain']),
        ap=dict(type='str', aliases=['app_profile', 'app_profile_name']),  # Not required for querying all objects
        tenant=dict(type='str', aliases=['tenant_name']),  # Not required for querying all objects
        description=dict(type='str', aliases=['descr']),
        priority=dict(type='str', choices=['level1', 'level2', 'level3', 'unspecified']),
        intra_epg_isolation=dict(choices=['enforced', 'unenforced']),
        fwd_control=dict(type='str', choices=['none', 'proxy-arp']),
        preferred_group=dict(type='bool'),
        state=dict(type='str', default='present', choices=['absent', 'present', 'query']),
        name_alias=dict(type='str'),
        monitoring_policy=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ['state', 'absent', ['ap', 'epg', 'tenant']],
            ['state', 'present', ['ap', 'epg', 'tenant']],
        ],
    )

    aci = ACIModule(module)

    epg = module.params.get('epg')
    bd = module.params.get('bd')
    description = module.params.get('description')
    priority = module.params.get('priority')
    intra_epg_isolation = module.params.get('intra_epg_isolation')
    fwd_control = module.params.get('fwd_control')
    preferred_group = aci.boolean(module.params.get('preferred_group'), 'include', 'exclude')
    state = module.params.get('state')
    tenant = module.params.get('tenant')
    ap = module.params.get('ap')
    name_alias = module.params.get('name_alias')
    monitoring_policy = module.params.get('monitoring_policy')

    child_configs = [
        dict(fvRsBd=dict(attributes=dict(tnFvBDName=bd))),
        dict(fvRsAEPgMonPol=dict(attributes=dict(tnMonEPGPolName=monitoring_policy)))
    ]

    aci.construct_url(
        root_class=dict(
            aci_class='fvTenant',
            aci_rn='tn-{0}'.format(tenant),
            module_object=tenant,
            target_filter={'name': tenant},
        ),
        subclass_1=dict(
            aci_class='fvAp',
            aci_rn='ap-{0}'.format(ap),
            module_object=ap,
            target_filter={'name': ap},
        ),
        subclass_2=dict(
            aci_class='fvAEPg',
            aci_rn='epg-{0}'.format(epg),
            module_object=epg,
            target_filter={'name': epg},
        ),
        child_classes=['fvRsBd', 'fvRsAEPgMonPol'],
    )

    aci.get_existing()

    if state == 'present':
        aci.payload(
            aci_class='fvAEPg',
            class_config=dict(
                name=epg,
                descr=description,
                prio=priority,
                pcEnfPref=intra_epg_isolation,
                fwdCtrl=fwd_control,
                prefGrMemb=preferred_group,
                nameAlias=name_alias,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class='fvAEPg')

        aci.post_config()

    elif state == 'absent':
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, 2018, Oracle and/or its affiliates.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
---
module: oci_vcn
short_description: Manage Virtual Cloud Networks(VCN) in OCI
description:
    - This module allows the user to create, delete and update virtual cloud networks(VCNs) in OCI.
      The complete Oracle Cloud Infrastructure Ansible Modules can be downloaded from
      U(https://github.com/oracle/oci-ansible-modules/releases).
options:
    cidr_block:
        description: The CIDR IP address block of the VCN. Required when creating a VCN with I(state=present).
        type: str
        required: false
    compartment_id:
        description: The OCID of the compartment to contain the VCN. Required when creating a VCN with I(state=present).
                     This option is mutually exclusive with I(vcn_id).
        type: str
    display_name:
        description: A user-friendly name. Does not have to be unique, and it's changeable.
        type: str
        aliases: [ 'name' ]
    dns_label:
        description: A DNS label for the VCN, used in conjunction with the VNIC's hostname and subnet's DNS label to
                     form a fully qualified domain name (FQDN) for each VNIC within this subnet (for example,
                     bminstance-1.subnet123.vcn1.oraclevcn.com). Not required to be unique, but it's a best practice
                     to set unique DNS labels for VCNs in your tenancy. Must be an alphanumeric string that begins
                     with a letter. The value cannot be changed.
        type: str
    state:
        description: Create or update a VCN with I(state=present). Use I(state=absent) to delete a VCN.
        type: str
        default: present
        choices: ['present', 'absent']
    vcn_id:
        description: The OCID of the VCN. Required when deleting a VCN with I(state=absent) or updating a VCN
                     with I(state=present). This option is mutually exclusive with I(compartment_id).
        type: str
        aliases: [ 'id' ]
author: "Rohit Chaware (@rohitChaware)"
extends_documentation_fragment:
- community.general.oracle
- community.general.oracle_creatable_resource
- community.general.oracle_wait_options
- community.general.oracle_tags

'''

EXAMPLES = """
- name: Create a VCN
  community.general.oci_vcn:
    cidr_block: '10.0.0.0/16'
    compartment_id: 'ocid1.compartment.oc1..xxxxxEXAMPLExxxxx'
    display_name: my_vcn
    dns_label: ansiblevcn

- name: Updates the specified VCN's display name
  community.general.oci_vcn:
    vcn_id: ocid1.vcn.oc1.phx.xxxxxEXAMPLExxxxx
    display_name: ansible_vcn

- name: Delete the specified VCN
  community.general.oci_vcn:
    vcn_id: ocid1.vcn.oc1.phx.xxxxxEXAMPLExxxxx
    state: absent
"""

RETURN = """
vcn:
    description: Information about the VCN
    returned: On successful create and update operation
    type: dict
    sample: {
            "cidr_block": "10.0.0.0/16",
            compartment_id": "ocid1.compartment.oc1..xxxxxEXAMPLExxxxx",
            "default_dhcp_options_id": "ocid1.dhcpoptions.oc1.phx.xxxxxEXAMPLExxxxx",
            "default_route_table_id": "ocid1.routetable.oc1.phx.xxxxxEXAMPLExxxxx",
            "default_security_list_id": "ocid1.securitylist.oc1.phx.xxxxxEXAMPLExxxxx",
            "display_name": "ansible_vcn",
            "dns_label": "ansiblevcn",
            "id": "ocid1.vcn.oc1.phx.xxxxxEXAMPLExxxxx",
            "lifecycle_state": "AVAILABLE",
            "time_created": "2017-11-13T20:22:40.626000+00:00",
            "vcn_domain_name": "ansiblevcn.oraclevcn.com"
        }
"""

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible_collections.community.general.plugins.module_utils.oracle import oci_utils

try:
    from oci.core.virtual_network_client import VirtualNetworkClient
    from oci.core.models import CreateVcnDetails
    from oci.core.models import UpdateVcnDetails

    HAS_OCI_PY_SDK = True
except ImportError:
    HAS_OCI_PY_SDK = False


def delete_vcn(virtual_network_client, module):
    result = oci_utils.delete_and_wait(
        resource_type="vcn",
        client=virtual_network_client,
        get_fn=virtual_network_client.get_vcn,
        kwargs_get={"vcn_id": module.params["vcn_id"]},
        delete_fn=virtual_network_client.delete_vcn,
        kwargs_delete={"vcn_id": module.params["vcn_id"]},
        module=module,
    )
    return result


def update_vcn(virtual_network_client, module):
    result = oci_utils.check_and_update_resource(
        resource_type="vcn",
        client=virtual_network_client,
        get_fn=virtual_network_client.get_vcn,
        kwargs_get={"vcn_id": module.params["vcn_id"]},
        update_fn=virtual_network_client.update_vcn,
        primitive_params_update=["vcn_id"],
        kwargs_non_primitive_update={UpdateVcnDetails: "update_vcn_details"},
        module=module,
        update_attributes=UpdateVcnDetails().attribute_map.keys(),
    )
    return result


def create_vcn(virtual_network_client, module):
    create_vcn_details = CreateVcnDetails()
    for attribute in create_vcn_details.attribute_map.keys():
        if attribute in module.params:
            setattr(create_vcn_details, attribute, module.params[attribute])

    result = oci_utils.create_and_wait(
        resource_type="vcn",
        create_fn=virtual_network_client.create_vcn,
        kwargs_create={"create_vcn_details": create_vcn_details},
        client=virtual_network_client,
        get_fn=virtual_network_client.get_vcn,
        get_param="vcn_id",
        module=module,
    )
    return result


def main():
    module_args = oci_utils.get_taggable_arg_spec(
        supports_create=True, supports_wait=True
    )
    module_args.update(
        dict(
            cidr_block=dict(type="str", required=False),
            compartment_id=dict(type="str", required=False),
            display_name=dict(type="str", required=False, aliases=["name"]),
            dns_label=dict(type="str", required=False),
            state=dict(
                type="str",
                required=False,
                default="present",
                choices=["absent", "present"],
            ),
            vcn_id=dict(type="str", required=False, aliases=["id"]),
        )
    )

    module = AnsibleModule(
        argument_spec=module_args,
        supports_check_mode=False,
        mutually_exclusive=[["compartment_id", "vcn_id"]],
    )

    if not HAS_OCI_PY_SDK:
        module.fail_json(msg=missing_required_lib("oci"))

    virtual_network_client = oci_utils.create_service_client(
        module, VirtualNetworkClient
    )

    exclude_attributes = {"display_name": True, "dns_label": True}
    state = module.params["state"]
    vcn_id = module.params["vcn_id"]

    if state == "absent":
        if vcn_id is not None:
            result = delete_vcn(virtual_network_client, module)
        else:
            module.fail_json(
                msg="Specify vcn_id with state as 'absent' to delete a VCN."
            )

    else:
        if vcn_id is not None:
            result = update_vcn(virtual_network_client, module)
        else:
            result = oci_utils.check_and_create_resource(
                resource_type="vcn",
                create_fn=create_vcn,
                kwargs_create={
                    "virtual_network_client": virtual_network_client,
                    "module": module,
                },
                list_fn=virtual_network_client.list_vcns,
                kwargs_list={"compartment_id": module.params["compartment_id"]},
                module=module,
                model=CreateVcnDetails(),
                exclude_attributes=exclude_attributes,
            )

    module.exit_json(**result)


if __name__ == "__main__":
    main()



# (c) 2017 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.cisco.iosxr.tests.unit.compat.mock import patch
from ansible_collections.cisco.iosxr.plugins.modules import iosxr_netconf
from ansible_collections.cisco.iosxr.tests.unit.modules.utils import (
    set_module_args,
)
from .iosxr_module import TestIosxrModule


class TestIosxrNetconfModule(TestIosxrModule):

    module = iosxr_netconf

    def setUp(self):
        super(TestIosxrNetconfModule, self).setUp()

        self.mock_get_config = patch(
            "ansible_collections.cisco.iosxr.plugins.modules.iosxr_netconf.get_config"
        )
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            "ansible_collections.cisco.iosxr.plugins.modules.iosxr_netconf.load_config"
        )
        self.load_config = self.mock_load_config.start()

    def tearDown(self):
        super(TestIosxrNetconfModule, self).tearDown()
        self.mock_get_config.stop()
        self.mock_load_config.stop()

    def test_iosxr_disable_netconf_service(self):
        self.get_config.return_value = """
        netconf-yang agent
            ssh
        !
        ssh server netconf vrf default
        """
        self.load_config.return_value = "dummy diff"
        set_module_args(
            dict(netconf_port=830, netconf_vrf="default", state="absent")
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "no netconf-yang agent ssh",
                "no ssh server netconf port 830",
                "no ssh server netconf vrf default",
            ],
        )

    def test_iosxr_enable_netconf_service(self):
        self.get_config.return_value = ""
        self.load_config.return_value = "dummy diff"
        set_module_args(
            dict(netconf_port=830, netconf_vrf="default", state="present")
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "netconf-yang agent ssh",
                "ssh server netconf port 830",
                "ssh server netconf vrf default",
            ],
        )

    def test_iosxr_change_netconf_port(self):
        self.get_config.return_value = """
        netconf-yang agent
            ssh
        !
        ssh server netconf vrf default
        """
        self.load_config.return_value = "dummy diff"
        set_module_args(dict(netconf_port=9000, state="present"))
        result = self.execute_module(changed=True)
        self.assertEqual(result["commands"], ["ssh server netconf port 9000"])

    def test_iosxr_change_netconf_vrf(self):
        self.get_config.return_value = """
        netconf-yang agent
            ssh
        !
        ssh server netconf vrf default
        """
        self.load_config.return_value = "dummy diff"
        set_module_args(dict(netconf_vrf="new_default", state="present"))
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"], ["ssh server netconf vrf new_default"]
        )



#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Copyright: (c) 2016, Adfinis SyGroup AG
# Tobias Rueetschi <tobias.ruetschi@adfinis-sygroup.ch>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: udm_share
author:
- Tobias Rüetschi (@keachi)
short_description: Manage samba shares on a univention corporate server
description:
    - "This module allows to manage samba shares on a univention corporate
       server (UCS).
       It uses the python API of the UCS to create a new object or edit it."
requirements:
    - Python >= 2.6
options:
    state:
        default: "present"
        choices: [ present, absent ]
        description:
            - Whether the share is present or not.
        type: str
    name:
        required: true
        description:
            - Name
        type: str
    host:
        required: false
        description:
            - Host FQDN (server which provides the share), e.g. C({{
              ansible_fqdn }}). Required if C(state=present).
        type: str
    path:
        required: false
        description:
            - Directory on the providing server, e.g. C(/home). Required if C(state=present).
        type: path
    sambaName:
        required: false
        description:
            - Windows name. Required if C(state=present).
        type: str
        aliases: [ samba_name ]
    ou:
        required: true
        description:
            - Organisational unit, inside the LDAP Base DN.
        type: str
    owner:
        default: '0'
        description:
            - Directory owner of the share's root directory.
        type: str
    group:
        default: '0'
        description:
            - Directory owner group of the share's root directory.
        type: str
    directorymode:
        default: '00755'
        description:
            - Permissions for the share's root directory.
        type: str
    root_squash:
        default: true
        description:
            - Modify user ID for root user (root squashing).
        type: bool
    subtree_checking:
        default: true
        description:
            - Subtree checking.
        type: bool
    sync:
        default: 'sync'
        description:
            - NFS synchronisation.
        type: str
    writeable:
        default: true
        description:
            - NFS write access.
        type: bool
    sambaBlockSize:
        description:
            - Blocking size.
        type: str
        aliases: [ samba_block_size ]
    sambaBlockingLocks:
        default: true
        description:
            - Blocking locks.
        type: bool
        aliases: [ samba_blocking_locks ]
    sambaBrowseable:
        description:
        - Show in Windows network environment.
        type: bool
        default: True
        aliases: [ samba_browsable ]
    sambaCreateMode:
        default: '0744'
        description:
            - File mode.
        type: str
        aliases: [ samba_create_mode ]
    sambaCscPolicy:
        default: 'manual'
        description:
            - Client-side caching policy.
        type: str
        aliases: [ samba_csc_policy ]
    sambaCustomSettings:
        default: []
        description:
            - Option name in smb.conf and its value.
        type: list
        aliases: [ samba_custom_settings ]
    sambaDirectoryMode:
        default: '0755'
        description:
            - Directory mode.
        type: str
        aliases: [ samba_directory_mode ]
    sambaDirectorySecurityMode:
        default: '0777'
        description:
            - Directory security mode.
        type: str
        aliases: [ samba_directory_security_mode ]
    sambaDosFilemode:
        default: false
        description:
            - Users with write access may modify permissions.
        type: bool
        aliases: [ samba_dos_filemode ]
    sambaFakeOplocks:
        default: false
        description:
            - Fake oplocks.
        type: bool
        aliases: [ samba_fake_oplocks ]
    sambaForceCreateMode:
        default: false
        description:
            - Force file mode.
        type: bool
        aliases: [ samba_force_create_mode ]
    sambaForceDirectoryMode:
        default: false
        description:
            - Force directory mode.
        type: bool
        aliases: [ samba_force_directory_mode ]
    sambaForceDirectorySecurityMode:
        default: false
        description:
            - Force directory security mode.
        type: bool
        aliases: [ samba_force_directory_security_mode ]
    sambaForceGroup:
        description:
            - Force group.
        type: str
        aliases: [ samba_force_group ]
    sambaForceSecurityMode:
        default: false
        description:
            - Force security mode.
        type: bool
        aliases: [ samba_force_security_mode ]
    sambaForceUser:
        description:
            - Force user.
        type: str
        aliases: [ samba_force_user ]
    sambaHideFiles:
        description:
            - Hide files.
        type: str
        aliases: [ samba_hide_files ]
    sambaHideUnreadable:
        default: false
        description:
            - Hide unreadable files/directories.
        type: bool
        aliases: [ samba_hide_unreadable ]
    sambaHostsAllow:
        default: []
        description:
            - Allowed host/network.
        type: list
        aliases: [ samba_hosts_allow ]
    sambaHostsDeny:
        default: []
        description:
            - Denied host/network.
        type: list
        aliases: [ samba_hosts_deny ]
    sambaInheritAcls:
        default: true
        description:
            - Inherit ACLs.
        type: bool
        aliases: [ samba_inherit_acls ]
    sambaInheritOwner:
        default: false
        description:
            - Create files/directories with the owner of the parent directory.
        type: bool
        aliases: [ samba_inherit_owner ]
    sambaInheritPermissions:
        default: false
        description:
            - Create files/directories with permissions of the parent directory.
        type: bool
        aliases: [ samba_inherit_permissions ]
    sambaInvalidUsers:
        description:
            - Invalid users or groups.
        type: str
        aliases: [ samba_invalid_users ]
    sambaLevel2Oplocks:
        default: true
        description:
            - Level 2 oplocks.
        type: bool
        aliases: [ samba_level_2_oplocks ]
    sambaLocking:
        default: true
        description:
            - Locking.
        type: bool
        aliases: [ samba_locking ]
    sambaMSDFSRoot:
        default: false
        description:
            - MSDFS root.
        type: bool
        aliases: [ samba_msdfs_root ]
    sambaNtAclSupport:
        default: true
        description:
            - NT ACL support.
        type: bool
        aliases: [ samba_nt_acl_support ]
    sambaOplocks:
        default: true
        description:
            - Oplocks.
        type: bool
        aliases: [ samba_oplocks ]
    sambaPostexec:
        description:
            - Postexec script.
        type: str
        aliases: [ samba_postexec ]
    sambaPreexec:
        description:
            - Preexec script.
        type: str
        aliases: [ samba_preexec ]
    sambaPublic:
        default: false
        description:
            - Allow anonymous read-only access with a guest user.
        type: bool
        aliases: [ samba_public ]
    sambaSecurityMode:
        default: '0777'
        description:
            - Security mode.
        type: str
        aliases: [ samba_security_mode ]
    sambaStrictLocking:
        default: 'Auto'
        description:
            - Strict locking.
        type: str
        aliases: [ samba_strict_locking ]
    sambaVFSObjects:
        description:
            - VFS objects.
        type: str
        aliases: [ samba_vfs_objects ]
    sambaValidUsers:
        description:
            - Valid users or groups.
        type: str
        aliases: [ samba_valid_users ]
    sambaWriteList:
        description:
            - Restrict write access to these users/groups.
        type: str
        aliases: [ samba_write_list ]
    sambaWriteable:
        default: true
        description:
            - Samba write access.
        type: bool
        aliases: [ samba_writeable ]
    nfs_hosts:
        default: []
        description:
            - Only allow access for this host, IP address or network.
        type: list
    nfsCustomSettings:
        default: []
        description:
            - Option name in exports file.
        type: list
        aliases: [ nfs_custom_settings ]
'''


EXAMPLES = '''
- name: Create a share named home on the server ucs.example.com with the path /home
  community.general.udm_share:
    name: home
    path: /home
    host: ucs.example.com
    sambaName: Home
'''


RETURN = '''# '''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.univention_umc import (
    umc_module_for_add,
    umc_module_for_edit,
    ldap_search,
    base_dn,
)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True,
                      type='str'),
            ou=dict(required=True,
                    type='str'),
            owner=dict(type='str',
                       default='0'),
            group=dict(type='str',
                       default='0'),
            path=dict(type='path',
                      default=None),
            directorymode=dict(type='str',
                               default='00755'),
            host=dict(type='str',
                      default=None),
            root_squash=dict(type='bool',
                             default=True),
            subtree_checking=dict(type='bool',
                                  default=True),
            sync=dict(type='str',
                      default='sync'),
            writeable=dict(type='bool',
                           default=True),
            sambaBlockSize=dict(type='str',
                                aliases=['samba_block_size'],
                                default=None),
            sambaBlockingLocks=dict(type='bool',
                                    aliases=['samba_blocking_locks'],
                                    default=True),
            sambaBrowseable=dict(type='bool',
                                 aliases=['samba_browsable'],
                                 default=True),
            sambaCreateMode=dict(type='str',
                                 aliases=['samba_create_mode'],
                                 default='0744'),
            sambaCscPolicy=dict(type='str',
                                aliases=['samba_csc_policy'],
                                default='manual'),
            sambaCustomSettings=dict(type='list',
                                     aliases=['samba_custom_settings'],
                                     default=[]),
            sambaDirectoryMode=dict(type='str',
                                    aliases=['samba_directory_mode'],
                                    default='0755'),
            sambaDirectorySecurityMode=dict(type='str',
                                            aliases=['samba_directory_security_mode'],
                                            default='0777'),
            sambaDosFilemode=dict(type='bool',
                                  aliases=['samba_dos_filemode'],
                                  default=False),
            sambaFakeOplocks=dict(type='bool',
                                  aliases=['samba_fake_oplocks'],
                                  default=False),
            sambaForceCreateMode=dict(type='bool',
                                      aliases=['samba_force_create_mode'],
                                      default=False),
            sambaForceDirectoryMode=dict(type='bool',
                                         aliases=['samba_force_directory_mode'],
                                         default=False),
            sambaForceDirectorySecurityMode=dict(type='bool',
                                                 aliases=['samba_force_directory_security_mode'],
                                                 default=False),
            sambaForceGroup=dict(type='str',
                                 aliases=['samba_force_group'],
                                 default=None),
            sambaForceSecurityMode=dict(type='bool',
                                        aliases=['samba_force_security_mode'],
                                        default=False),
            sambaForceUser=dict(type='str',
                                aliases=['samba_force_user'],
                                default=None),
            sambaHideFiles=dict(type='str',
                                aliases=['samba_hide_files'],
                                default=None),
            sambaHideUnreadable=dict(type='bool',
                                     aliases=['samba_hide_unreadable'],
                                     default=False),
            sambaHostsAllow=dict(type='list',
                                 aliases=['samba_hosts_allow'],
                                 default=[]),
            sambaHostsDeny=dict(type='list',
                                aliases=['samba_hosts_deny'],
                                default=[]),
            sambaInheritAcls=dict(type='bool',
                                  aliases=['samba_inherit_acls'],
                                  default=True),
            sambaInheritOwner=dict(type='bool',
                                   aliases=['samba_inherit_owner'],
                                   default=False),
            sambaInheritPermissions=dict(type='bool',
                                         aliases=['samba_inherit_permissions'],
                                         default=False),
            sambaInvalidUsers=dict(type='str',
                                   aliases=['samba_invalid_users'],
                                   default=None),
            sambaLevel2Oplocks=dict(type='bool',
                                    aliases=['samba_level_2_oplocks'],
                                    default=True),
            sambaLocking=dict(type='bool',
                              aliases=['samba_locking'],
                              default=True),
            sambaMSDFSRoot=dict(type='bool',
                                aliases=['samba_msdfs_root'],
                                default=False),
            sambaName=dict(type='str',
                           aliases=['samba_name'],
                           default=None),
            sambaNtAclSupport=dict(type='bool',
                                   aliases=['samba_nt_acl_support'],
                                   default=True),
            sambaOplocks=dict(type='bool',
                              aliases=['samba_oplocks'],
                              default=True),
            sambaPostexec=dict(type='str',
                               aliases=['samba_postexec'],
                               default=None),
            sambaPreexec=dict(type='str',
                              aliases=['samba_preexec'],
                              default=None),
            sambaPublic=dict(type='bool',
                             aliases=['samba_public'],
                             default=False),
            sambaSecurityMode=dict(type='str',
                                   aliases=['samba_security_mode'],
                                   default='0777'),
            sambaStrictLocking=dict(type='str',
                                    aliases=['samba_strict_locking'],
                                    default='Auto'),
            sambaVFSObjects=dict(type='str',
                                 aliases=['samba_vfs_objects'],
                                 default=None),
            sambaValidUsers=dict(type='str',
                                 aliases=['samba_valid_users'],
                                 default=None),
            sambaWriteList=dict(type='str',
                                aliases=['samba_write_list'],
                                default=None),
            sambaWriteable=dict(type='bool',
                                aliases=['samba_writeable'],
                                default=True),
            nfs_hosts=dict(type='list',
                           default=[]),
            nfsCustomSettings=dict(type='list',
                                   aliases=['nfs_custom_settings'],
                                   default=[]),
            state=dict(default='present',
                       choices=['present', 'absent'],
                       type='str')
        ),
        supports_check_mode=True,
        required_if=([
            ('state', 'present', ['path', 'host', 'sambaName'])
        ])
    )
    name = module.params['name']
    state = module.params['state']
    changed = False
    diff = None

    obj = list(ldap_search(
        '(&(objectClass=univentionShare)(cn={0}))'.format(name),
        attr=['cn']
    ))

    exists = bool(len(obj))
    container = 'cn=shares,ou={0},{1}'.format(module.params['ou'], base_dn())
    dn = 'cn={0},{1}'.format(name, container)

    if state == 'present':
        try:
            if not exists:
                obj = umc_module_for_add('shares/share', container)
            else:
                obj = umc_module_for_edit('shares/share', dn)

            module.params['printablename'] = '{0} ({1})'.format(name, module.params['host'])
            for k in obj.keys():
                if module.params[k] is True:
                    module.params[k] = '1'
                elif module.params[k] is False:
                    module.params[k] = '0'
                obj[k] = module.params[k]

            diff = obj.diff()
            if exists:
                for k in obj.keys():
                    if obj.hasChanged(k):
                        changed = True
            else:
                changed = True
            if not module.check_mode:
                if not exists:
                    obj.create()
                elif changed:
                    obj.modify()
        except Exception as err:
            module.fail_json(
                msg='Creating/editing share {0} in {1} failed: {2}'.format(
                    name,
                    container,
                    err,
                )
            )

    if state == 'absent' and exists:
        try:
            obj = umc_module_for_edit('shares/share', dn)
            if not module.check_mode:
                obj.remove()
            changed = True
        except Exception as err:
            module.fail_json(
                msg='Removing share {0} in {1} failed: {2}'.format(
                    name,
                    container,
                    err,
                )
            )

    module.exit_json(
        changed=changed,
        name=name,
        diff=diff,
        container=container
    )


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for iosxr_static_routes
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: iosxr_static_routes
short_description: Static routes resource module
description:
- This module manages static routes on devices running Cisco IOS-XR.
version_added: 1.0.0
author: Nilashish Chakraborty (@NilashishC)
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the IOS-XR device
      by executing the command B(show running-config router static).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A dictionary of static route options.
    type: list
    elements: dict
    suboptions:
      vrf:
        description:
        - The VRF to which the static route(s) belong.
        type: str
      address_families:
        description: A dictionary specifying the address family to which the static
          route(s) belong.
        type: list
        elements: dict
        suboptions:
          afi:
            description:
            - Specifies the top level address family indicator.
            type: str
            choices:
            - ipv4
            - ipv6
            required: true
          safi:
            description:
            - Specifies the subsequent address family indicator.
            type: str
            choices:
            - unicast
            - multicast
            required: true
          routes:
            description: A dictionary that specifies the static route configurations.
            elements: dict
            type: list
            suboptions:
              dest:
                description:
                - An IPv4 or IPv6 address in CIDR notation that specifies the destination
                  network for the static route.
                type: str
                required: true
              next_hops:
                description:
                - Next hops to the specified destination.
                type: list
                elements: dict
                suboptions:
                  forward_router_address:
                    description:
                    - The IP address of the next hop that can be used to reach the
                      destination network.
                    type: str
                  interface:
                    description:
                    - The interface to use to reach the destination.
                    type: str
                  dest_vrf:
                    description:
                    - The destination VRF.
                    type: str
                  admin_distance:
                    description:
                    - The administrative distance for this static route.
                    - Refer to vendor documentation for valid values.
                    type: int
                  metric:
                    description:
                    - Specifes the metric for this static route.
                    - Refer to vendor documentation for valid values.
                    type: int
                  description:
                    description:
                    - Specifies the description for this static route.
                    type: str
                  vrflabel:
                    description:
                    - Specifies the VRF label for this static route.
                    - Refer to vendor documentation for valid values.
                    type: int
                  tag:
                    description:
                    - Specifies a numeric tag for this static route.
                    - Refer to vendor documentation for valid values.
                    type: int
                  track:
                    description:
                    - Specifies the object to be tracked.
                    - This enables object tracking for static routes.
                    type: str
                  tunnel_id:
                    description:
                    - Specifies a tunnel id for the route.
                    - Refer to vendor documentation for valid values.
                    type: int
  state:
    description:
    - The state the configuration should be left in.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - gathered
    - rendered
    - parsed
    default: merged

"""
EXAMPLES = """

# Using merged

# Before state
# -------------
# RP/0/RP0/CPU0:ios#show running-config router static
# Sat Feb 22 07:46:30.089 UTC
# % No such configuration item(s)
#
- name: Merge the provided configuration with the exisiting running configuration
  cisco.iosxr.iosxr_static_routes:
    config:
    - address_families:
      - afi: ipv4
        safi: unicast
        routes:
        - dest: 192.0.2.16/28
          next_hops:
          - forward_router_address: 192.0.2.10
            interface: FastEthernet0/0/0/1
            description: LAB
            metric: 120
            tag: 10

          - interface: FastEthernet0/0/0/5
            track: ip_sla_1

        - dest: 192.0.2.32/28
          next_hops:
          - forward_router_address: 192.0.2.11
            admin_distance: 100

      - afi: ipv6
        safi: unicast
        routes:
        - dest: 2001:db8:1000::/36
          next_hops:
          - interface: FastEthernet0/0/0/7
            description: DC

          - interface: FastEthernet0/0/0/8
            forward_router_address: 2001:db8:2000:2::1

    - vrf: DEV_SITE
      address_families:
      - afi: ipv4
        safi: unicast
        routes:
        - dest: 192.0.2.48/28
          next_hops:
          - forward_router_address: 192.0.2.12
            description: DEV
            dest_vrf: test_1

        - dest: 192.0.2.80/28
          next_hops:
          - interface: FastEthernet0/0/0/2
            forward_router_address: 192.0.2.14
            dest_vrf: test_1
            track: ip_sla_2
            vrflabel: 124
    state: merged

# After state
# -------------
# RP/0/RP0/CPU0:ios#show running-config router static
# Sat Feb 22 07:49:11.754 UTC
# router static
#  address-family ipv4 unicast
#   192.0.2.16/28 FastEthernet0/0/0/1 192.0.2.10 tag 10 description LAB metric 120
#   192.0.2.16/28 FastEthernet0/0/0/5 track ip_sla_1
#   192.0.2.32/28 192.0.2.11 100
#  !
#  address-family ipv6 unicast
#   2001:db8:1000::/36 FastEthernet0/0/0/7 description DC
#   2001:db8:1000::/36 FastEthernet0/0/0/8 2001:db8:2000:2::1
#  !
#  vrf DEV_SITE
#   address-family ipv4 unicast
#    192.0.2.48/28 vrf test_1 192.0.2.12 description DEV
#    192.0.2.80/28 vrf test_1 FastEthernet0/0/0/2 192.0.2.14 vrflabel 124 track ip_sla_2
#   !
#  !
# !

# Using merged to update existing static routes

# Before state
# -------------
# RP/0/RP0/CPU0:ios#show running-config router static
# Sat Feb 22 07:49:11.754 UTC
# router static
#  address-family ipv4 unicast
#   192.0.2.16/28 FastEthernet0/0/0/1 192.0.2.10 tag 10 description LAB metric 120
#   192.0.2.16/28 FastEthernet0/0/0/5 track ip_sla_1
#   192.0.2.32/28 192.0.2.11 100
#  !
#  address-family ipv6 unicast
#   2001:db8:1000::/36 FastEthernet0/0/0/7 description DC
#   2001:db8:1000::/36 FastEthernet0/0/0/8 2001:db8:2000:2::1
#  !
#  vrf DEV_SITE
#   address-family ipv4 unicast
#    192.0.2.48/28 vrf test_1 192.0.2.12 description DEV
#    192.0.2.80/28 vrf test_1 FastEthernet0/0/0/2 192.0.2.14 vrflabel 124 track ip_sla_2
#   !
#  !
# !

- name: Update existing static routes configuration using merged
  cisco.iosxr.iosxr_static_routes:
    config:
    - vrf: DEV_SITE
      address_families:
      - afi: ipv4
        safi: unicast
        routes:
        - dest: 192.0.2.48/28
          next_hops:
          - forward_router_address: 192.0.2.12
            vrflabel: 2301
            dest_vrf: test_1

        - dest: 192.0.2.80/28
          next_hops:
          - interface: FastEthernet0/0/0/2
            forward_router_address: 192.0.2.14
            dest_vrf: test_1
            description: rt_test_1
    state: merged

# After state
# -------------
# RP/0/RP0/CPU0:ios#show running-config router static
# Sat Feb 22 07:49:11.754 UTC
# router static
#  address-family ipv4 unicast
#   192.0.2.16/28 FastEthernet0/0/0/1 192.0.2.10 tag 10 description LAB metric 120
#   192.0.2.16/28 FastEthernet0/0/0/5 track ip_sla_1
#   192.0.2.32/28 192.0.2.11 100
#  !
#  address-family ipv6 unicast
#   2001:db8:1000::/36 FastEthernet0/0/0/7 description DC
#   2001:db8:1000::/36 FastEthernet0/0/0/8 2001:db8:2000:2::1
#  !
#  vrf DEV_SITE
#   address-family ipv4 unicast
#    192.0.2.48/28 vrf test_1 192.0.2.12 description DEV vrflabel 2301
#    192.0.2.80/28 vrf test_1 192.0.2.14 FastEthernet0/0/0/2 description rt_test_1 track ip_sla_2 vrflabel 124
#   !
#  !
# !

# Using replaced to replace all next hop entries for a single destination network

# Before state
# --------------

# RP/0/RP0/CPU0:ios#sh running-config router static
# Sat Feb 22 07:59:08.669 UTC
# router static
#  address-family ipv4 unicast
#   192.0.2.16/28 FastEthernet0/0/0/1 192.0.2.10 tag 10 description LAB metric 120
#   192.0.2.16/28 FastEthernet0/0/0/5 track ip_sla_1
#   192.0.2.32/28 192.0.2.11 100
#  !
#  address-family ipv6 unicast
#   2001:db8:1000::/36 FastEthernet0/0/0/7 description DC
#   2001:db8:1000::/36 FastEthernet0/0/0/8 2001:db8:2000:2::1
#  !
#  vrf DEV_SITE
#   address-family ipv4 unicast
#    192.0.2.48/28 vrf test_1 192.0.2.12 description DEV
#    192.0.2.48/28 GigabitEthernet0/0/0/1 192.0.3.24 vrflabel 2302
#    192.0.2.80/28 vrf test_1 FastEthernet0/0/0/2 192.0.2.14 vrflabel 124 track ip_sla_2
#   !
#  !
# !

- name: Replace device configurations of static routes with provided configurations
  cisco.iosxr.iosxr_static_routes:
    config:
    - vrf: DEV_SITE
      address_families:
      - afi: ipv4
        safi: unicast
        routes:
        - dest: 192.0.2.48/28
          next_hops:
          - forward_router_address: 192.0.2.15
            interface: FastEthernet0/0/0/3
            description: DEV_NEW
            dest_vrf: dev_test_2
    state: replaced

# After state
# ------------
# RP/0/RP0/CPU0:ios#sh running-config router static
# Sat Feb 22 08:04:07.085 UTC
# router static
#  address-family ipv4 unicast
#   192.0.2.16/28 FastEthernet0/0/0/1 192.0.2.10 tag 10 description LAB metric 120
#   192.0.2.16/28 FastEthernet0/0/0/5 track ip_sla_1
#   192.0.2.32/28 192.0.2.11 100
#  !
#  address-family ipv6 unicast
#   2001:db8:1000::/36 FastEthernet0/0/0/7 description DC
#   2001:db8:1000::/36 FastEthernet0/0/0/8 2001:db8:2000:2::1
#  !
#  vrf DEV_SITE
#   address-family ipv4 unicast
#    192.0.2.48/28 vrf dev_test_2 FastEthernet0/0/0/3 192.0.2.15 description DEV_NEW
#    192.0.2.80/28 vrf test_1 FastEthernet0/0/0/2 192.0.2.14 vrflabel 124 track ip_sla_2
#   !
#  !
# !

# Using overridden to override all static route entries on the device

# Before state
# -------------
# RP/0/RP0/CPU0:ios#sh running-config router static
# Sat Feb 22 07:59:08.669 UTC
# router static
#  address-family ipv4 unicast
#   192.0.2.16/28 FastEthernet0/0/0/1 192.0.2.10 tag 10 description LAB metric 120
#   192.0.2.16/28 FastEthernet0/0/0/5 track ip_sla_1
#   192.0.2.32/28 192.0.2.11 100
#  !
#  address-family ipv6 unicast
#   2001:db8:1000::/36 FastEthernet0/0/0/7 description DC
#   2001:db8:1000::/36 FastEthernet0/0/0/8 2001:db8:2000:2::1
#  !
#  vrf DEV_SITE
#   address-family ipv4 unicast
#    192.0.2.48/28 vrf test_1 192.0.2.12 description DEV
#    192.0.2.48/28 GigabitEthernet0/0/0/1 192.0.3.24 vrflabel 2302
#    192.0.2.80/28 vrf test_1 FastEthernet0/0/0/2 192.0.2.14 vrflabel 124 track ip_sla_2
#   !
#  !
# !

- name: Overridde all static routes configuration with provided configuration
  cisco.iosxr.iosxr_static_routes:
    config:
    - vrf: DEV_NEW
      address_families:
      - afi: ipv4
        safi: unicast
        routes:
        - dest: 192.0.2.48/28
          next_hops:
          - forward_router_address: 192.0.2.15
            interface: FastEthernet0/0/0/3
            description: DEV1
      - afi: ipv6
        safi: unicast
        routes:
        - dest: 2001:db8:3000::/36
          next_hops:
          - interface: FastEthernet0/0/0/4
            forward_router_address: 2001:db8:2000:2::2
            description: PROD1
            track: ip_sla_1
    state: overridden

# After state
# -------------
# RP/0/RP0/CPU0:ios#sh running-config router static
# Sat Feb 22 08:07:41.516 UTC
# router static
#  vrf DEV_NEW
#   address-family ipv4 unicast
#    192.0.2.48/28 FastEthernet0/0/0/3 192.0.2.15 description DEV1
#   !
#   address-family ipv6 unicast
#    2001:db8:3000::/36 FastEthernet0/0/0/4 2001:db8:2000:2::2 description PROD1 track ip_sla_1
#   !
#  !
# !

# Using deleted to delete all destination network entries under a single AFI

# Before state
# -------------
# RP/0/RP0/CPU0:ios#sh running-config router static
# Sat Feb 22 07:59:08.669 UTC
# router static
#  address-family ipv4 unicast
#   192.0.2.16/28 FastEthernet0/0/0/1 192.0.2.10 tag 10 description LAB metric 120
#   192.0.2.16/28 FastEthernet0/0/0/5 track ip_sla_1
#   192.0.2.32/28 192.0.2.11 100
#  !
#  address-family ipv6 unicast
#   2001:db8:1000::/36 FastEthernet0/0/0/7 description DC
#   2001:db8:1000::/36 FastEthernet0/0/0/8 2001:db8:2000:2::1
#  !
#  vrf DEV_SITE
#   address-family ipv4 unicast
#    192.0.2.48/28 vrf test_1 192.0.2.12 description DEV
#    192.0.2.48/28 GigabitEthernet0/0/0/1 192.0.3.24 vrflabel 2302
#    192.0.2.80/28 vrf test_1 FastEthernet0/0/0/2 192.0.2.14 vrflabel 124 track ip_sla_2
#   !
#  !
# !

- name: Delete all destination network entries under a single AFI
  cisco.iosxr.iosxr_static_routes:
    config:
    - vrf: DEV_SITE
      address_families:
      - afi: ipv4
        safi: unicast
    state: deleted

# After state
# ------------

# RP/0/RP0/CPU0:ios#sh running-config router static
# Sat Feb 22 08:16:41.464 UTC
# router static
#  address-family ipv4 unicast
#   192.0.2.16/28 FastEthernet0/0/0/1 192.0.2.10 tag 10 description LAB metric 120
#   192.0.2.16/28 FastEthernet0/0/0/5 track ip_sla_1
#   192.0.2.32/28 192.0.2.11 100
#  !
#  address-family ipv6 unicast
#   2001:db8:1000::/36 FastEthernet0/0/0/7 description DC
#   2001:db8:1000::/36 FastEthernet0/0/0/8 2001:db8:2000:2::1
#  !
#  vrf DEV_SITE
#  !
# !

# Using deleted to remove all static route entries from the device

# Before state
# -------------
# RP/0/RP0/CPU0:ios#sh running-config router static
# Sat Feb 22 07:59:08.669 UTC
# router static
#  address-family ipv4 unicast
#   192.0.2.16/28 FastEthernet0/0/0/1 192.0.2.10 tag 10 description LAB metric 120
#   192.0.2.16/28 FastEthernet0/0/0/5 track ip_sla_1
#   192.0.2.32/28 192.0.2.11 100
#  !
#  address-family ipv6 unicast
#   2001:db8:1000::/36 FastEthernet0/0/0/7 description DC
#   2001:db8:1000::/36 FastEthernet0/0/0/8 2001:db8:2000:2::1
#  !
#  vrf DEV_SITE
#   address-family ipv4 unicast
#    192.0.2.48/28 vrf test_1 192.0.2.12 description DEV
#    192.0.2.48/28 GigabitEthernet0/0/0/1 192.0.3.24 vrflabel 2302
#    192.0.2.80/28 vrf test_1 FastEthernet0/0/0/2 192.0.2.14 vrflabel 124 track ip_sla_2
#   !
#  !
# !

- name: Delete static routes configuration
  cisco.iosxr.iosxr_static_routes:
    state: deleted

# After state
# ------------
# RP/0/RP0/CPU0:ios#sh running-config router static
# Sat Feb 22 08:50:43.038 UTC
# % No such configuration item(s)

# Using gathered to gather static route facts from the device

- name: Gather static routes facts from the device using iosxr_static_routes module
  cisco.iosxr.iosxr_static_routes:
    state: gathered

# Task output (redacted)
# -----------------------
# "gathered": [
#    {
#        "address_families": [
#            {
#                "afi": "ipv4",
#                "routes": [
#                    {
#                        "dest": "192.0.2.16/28",
#                        "next_hops": [
#                            {
#                                "description": "LAB",
#                                "forward_router_address": "192.0.2.10",
#                                "interface": "FastEthernet0/0/0/1",
#                                "metric": 120,
#                                "tag": 10
#                            },
#                            {
#                                "interface": "FastEthernet0/0/0/5",
#                                "track": "ip_sla_1"
#                            }
#                        ]
#                    },
#                    {
#                        "dest": "192.0.2.32/28",
#                        "next_hops": [
#                            {
#                                "admin_distance": 100,
#                                "forward_router_address": "192.0.2.11"
#                            }
#                        ]
#                    }
#                ],
#                "safi": "unicast"
#            },
#            {
#                "afi": "ipv6",
#                "routes": [
#                    {
#                        "dest": "2001:db8:1000::/36",
#                        "next_hops": [
#                            {
#                                "description": "DC",
#                                "interface": "FastEthernet0/0/0/7"
#                            },
#                            {
#                                "forward_router_address": "2001:db8:2000:2::1",
#                                "interface": "FastEthernet0/0/0/8"
#                            }
#                        ]
#                    }
#                ],
#                "safi": "unicast"
#            }
#        ]
#    },
#    {
#        "address_families": [
#            {
#                "afi": "ipv4",
#                "routes": [
#                    {
#                        "dest": "192.0.2.48/28",
#                        "next_hops": [
#                            {
#                                "description": "DEV",
#                                "dest_vrf": "test_1",
#                                "forward_router_address": "192.0.2.12"
#                            },
#                            {
#                                "forward_router_address": "192.0.3.24",
#                                "interface": "GigabitEthernet0/0/0/1",
#                                "vrflabel": 2302
#                            }
#                        ]
#                    },
#                    {
#                        "dest": "192.0.2.80/28",
#                        "next_hops": [
#                            {
#                                "dest_vrf": "test_1",
#                                "forward_router_address": "192.0.2.14",
#                                "interface": "FastEthernet0/0/0/2",
#                                "track": "ip_sla_2",
#                                "vrflabel": 124
#                            }
#                        ]
#                    }
#                ],
#                "safi": "unicast"
#            }
#        ],
#        "vrf": "DEV_SITE"
#    }
#  ]

# Using rendered

- name: Render platform specific commands (without connecting to the device)
  cisco.iosxr.iosxr_static_routes:
  config:
  - vrf: DEV_SITE
    address_families:
    - afi: ipv4
      safi: unicast
      routes:
      - dest: 192.0.2.48/28
        next_hops:
        - forward_router_address: 192.0.2.12
          description: DEV
          dest_vrf: test_1

      - dest: 192.0.2.80/28
        next_hops:
        - interface: FastEthernet0/0/0/2
          forward_router_address: 192.0.2.14
          dest_vrf: test_1
          track: ip_sla_2
          vrflabel: 124

# Task Output (redacted)
# -----------------------
# "rendered": [
#    "router static"s,
#    "vrf DEV_SITE",
#    "address-family ipv4 unicast",
#    "192.0.2.48/28 vrf test_1 192.0.2.12 description DEV",
#    "192.0.2.80/28 vrf test_1 192.0.2.14 FastEthernet0/0/0/2 track ip_sla_2 vrflabel 124"

# Using parsed

# parsed.cfg
# ------------
# Fri Nov 29 21:10:41.896 UTC
# router static
#  address-family ipv4 unicast
#   192.0.2.16/28 FastEthernet0/0/0/1 192.0.2.10 tag 10 description LAB metric 120
#   192.0.2.16/28 FastEthernet0/0/0/5 track ip_sla_1
#   192.0.2.32/28 192.0.2.11 100
#  !
#  address-family ipv6 unicast
#   2001:db8:1000::/36 FastEthernet0/0/0/7 description DC
#   2001:db8:1000::/36 FastEthernet0/0/0/8 2001:db8:2000:2::1
#  !
#  vrf DEV_SITE
#   address-family ipv4 unicast
#    192.0.2.48/28 vrf test_1 192.0.2.12 description DEV
#    192.0.2.80/28 vrf test_1 FastEthernet0/0/0/2 192.0.2.14 vrflabel 124 track ip_sla_2
#   !
#  !
# !

- name: Use parsed state to convert externally supplied device specific static routes
    commands to structured format
  cisco.iosxr.iosxr_static_routes:
    running_config: "{{ lookup('file', '../../fixtures/parsed.cfg') }}"
    state: parsed

# Task output (redacted)
# -----------------------
# "parsed": [
#        {
#            "address_families": [
#                {
#                    "afi": "ipv4",
#                    "routes": [
#                        {
#                            "dest": "192.0.2.16/28",
#                            "next_hops": [
#                                {
#                                    "description": "LAB",
#                                    "forward_router_address": "192.0.2.10",
#                                    "interface": "FastEthernet0/0/0/1",
#                                    "metric": 120,
#                                    "tag": 10
#                                },
#                                {
#                                    "interface": "FastEthernet0/0/0/5",
#                                    "track": "ip_sla_1"
#                                }
#                            ]
#                        },
#                        {
#                            "dest": "192.0.2.32/28",
#                            "next_hops": [
#                                {
#                                    "admin_distance": 100,
#                                    "forward_router_address": "192.0.2.11"
#                                }
#                            ]
#                        }
#                    ],
#                    "safi": "unicast"
#                },
#                {
#                    "afi": "ipv6",
#                    "routes": [
#                        {
#                            "dest": "2001:db8:1000::/36",
#                            "next_hops": [
#                                {
#                                    "description": "DC",
#                                    "interface": "FastEthernet0/0/0/7"
#                                },
#                                {
#                                    "forward_router_address": "2001:db8:2000:2::1",
#                                    "interface": "FastEthernet0/0/0/8"
#                                }
#                            ]
#                        }
#                    ],
#                    "safi": "unicast"
#                }
#            ]
#        },
#        {
#            "address_families": [
#                {
#                    "afi": "ipv4",
#                    "routes": [
#                        {
#                            "dest": "192.0.2.48/28",
#                            "next_hops": [
#                                {
#                                    "description": "DEV",
#                                    "dest_vrf": "test_1",
#                                    "forward_router_address": "192.0.2.12"
#                                }
#                            ]
#                        },
#                        {
#                            "dest": "192.0.2.80/28",
#                            "next_hops": [
#                                {
#                                    "dest_vrf": "test_1",
#                                    "forward_router_address": "192.0.2.14",
#                                    "interface": "FastEthernet0/0/0/2",
#                                    "track": "ip_sla_2",
#                                    "vrflabel": 124
#                                }
#                            ]
#                        }
#                    ],
#                    "safi": "unicast"
#                }
#            ],
#            "vrf": "DEV_SITE"
#        }
#    ]
# }
"""
RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The resulting configuration model invocation.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample:
    - router static
    - vrf dev_site
    - address-family ipv4 unicast
    - 192.0.2.48/28 192.0.2.12 FastEthernet0/0/0/1 track ip_sla_10 description dev1
    - address-family ipv6 unicast
    - no 2001:db8:1000::/36
    - 2001:db8:3000::/36 2001:db8:2000:2::2 FastEthernet0/0/0/4 track ip_sla_11 description prod1
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.static_routes.static_routes import (
    Static_routesArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.static_routes.static_routes import (
    Static_routes,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]
    mutually_exclusive = [("config", "running_config")]

    module = AnsibleModule(
        argument_spec=Static_routesArgs.argument_spec,
        required_if=required_if,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )

    result = Static_routes(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()
